/*
*   umasip is a Videoconference program that uses the SIP protocol (RFC 3261)
*   Copyright (C) 2004 Pablo Montoro Escao
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

/**
 * <p>Programa: umasip</p>
 * <p>Copyright: Copyright GNU/GPL (c) 2004 Pablo Montoro Escao</p>
 * @author Pablo Montoro Escao
 * @version 1.00
 * @since SDK 1.4.2.
 */

public class Interfaz extends JFrame {

    public final int VERSION=umasip.VERSION;
    final String TITULO=new String("Java Aplication"); //Titulo por defecto
    final Dimension DIMENSION=new Dimension(400,200); //Dimension por defecto
    private final String RETURN=new String("\n"); //Cadena para fin de linea

    private Image fondo=getToolkit().getImage //Imagen de Fondo
                              ("img"+java.io.File.separator+"fondo.png");

    public String uriRemoto; //Direccion a llamar sip:user@host
    public boolean solicitaLlamada=false; //Indica si se solicita Llamada
    public boolean solicitaColgar=false; //Indica si se solicita colgar llamada
    public boolean solicitaPref=false; //Indica solicitud cambiar preferencias
    public boolean solicitaTX=false; //Indica solicitud informacion Transmision
    public boolean hayLlamada=false; //Indica si existe una llamada en curso
    public boolean fin=false; //Indica si se ha solicitado el fin del programa
    public Idioma idioma; //Modulo de idioma
    private int idiomaListener; //Valor del idioma que adoptara cada Listener

    private MenuBar barra=new MenuBar(); //Menu de opciones
    private Menu menuLlamada,menuConf,menuIdioma,menuAyuda,ayudas;
    private MenuItem nueva,colgar,salir,infoTX,preferencias,licencia,acercade;
    private CheckboxMenuItem verSIP,autoresponder;
    private MenuItem[] ayuda=new MenuItem[12];

    private JTextArea textoSIP; //Area de texto para informacion debug de SIP
    private JScrollPane ventanaTexto;
    private String sipDebug=""; //Informacion debug del protocolo SIP
    private int indexAyuda; //Indice de la ayuda solicitada
    private boolean hayJMF=true; //Indica si se pueden usar las opciones con JMF

    /**
     * Constructor del Interfaz. Se toman dimensiones por defecto.
     */
    public Interfaz() {
        cargarImagen();
        init(TITULO,
             (fondo!=null)?fondo.getWidth(this):DIMENSION.width,
             (fondo!=null)?fondo.getHeight(this):DIMENSION.height);
    }

    /**
     * Constructor del Interfaz. Se toman dimensiones por defecto.
     * @param titulo Nombre que aparecer en el Interfaz
     */
    public Interfaz(String titulo) {
        cargarImagen();
        init(titulo,
             (fondo!=null)?fondo.getWidth(this):DIMENSION.width,
             (fondo!=null)?fondo.getHeight(this):DIMENSION.height);
    }

    /**
     * Constructor del Interfaz
     * @param titulo Nombre que aparecer en el Interfaz
     * @param dimension Dimensiones del Interfaz
     */
    public Interfaz(String titulo,Dimension dimension) {
        cargarImagen();
        init(titulo,dimension.width,dimension.height);
    }

    /**
     * Constructor del Interfaz
     * @param titulo Nombre que aparecer en el Interfaz
     * @param width Ancho del Interfaz
     * @param height Alto del Interfaz
     */
    public Interfaz(String titulo,int width,int height) {
        cargarImagen();
        init(titulo,width,height);
    }

    //Metodo constructor general
    private void init(String titulo,int width,int height) {
        Dimension screenSize=Toolkit.getDefaultToolkit().getScreenSize();

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                fin=true;
            }
            public void windowClosed(WindowEvent e) {
                fin=true;
            }
        });

        addWindowStateListener(new WindowAdapter() {
            public void windowStateChanged(WindowEvent e) {
                repaint();
            }
        });

        idioma=new Idioma(); //Se carga el modulo de idioma

        //Inicializacion de componentes del menu
        menuLlamada=new Menu(idioma.texto(1)); //Llamada
        nueva=new MenuItem(idioma.texto(11)); //NuevaLlamada
        nueva.setEnabled(true);
        menuLlamada.add(nueva);
        colgar=new MenuItem(idioma.texto(12)); //Colgar
        colgar.setEnabled(false);
        menuLlamada.add(colgar);
        menuLlamada.addSeparator(); //----------
        salir=new MenuItem(idioma.texto(19)); //Salir
        menuLlamada.add(salir);
        barra.add(menuLlamada);

        menuConf=new Menu(idioma.texto(2)); //Configuracion
        verSIP=new CheckboxMenuItem(idioma.texto(21)); //verSIP
        verSIP.setState(true);
        menuConf.add(verSIP);
        infoTX=new MenuItem(idioma.texto(22)); //infoTX
        menuConf.add(infoTX);
        autoresponder=new CheckboxMenuItem(idioma.texto(23)); //Autoresponder
        autoresponder.setState(false);
        menuConf.add(autoresponder);
        menuConf.addSeparator(); //----------
        preferencias=new MenuItem(idioma.texto(29)); //Preferencias
        preferencias.setEnabled(true);
        menuConf.add(preferencias);
        barra.add(menuConf);

        menuIdioma=new Menu(idioma.texto(3)); //Idioma
        MenuItem[] idiomas=new MenuItem[idioma.disponibles.nombre.length];
        for (int i=0; i<idioma.disponibles.nombre.length; i++) {
            idiomas[i]=new MenuItem(idioma.disponibles.nombre[i]);
            menuIdioma.add(idiomas[i]);
            idiomaListener=i;
            idiomas[i].addActionListener(new ActionListener() {
                int miIdioma=idiomaListener; //Cada objeto captura su Idioma

                public void actionPerformed(ActionEvent e) {
                    idioma.cargar(miIdioma);
                    cambioIdioma();
                }
            });
        }
        barra.add(menuIdioma);

        menuAyuda=new Menu(idioma.texto(9)); //Ayuda
        ayudas=new Menu(idioma.texto(91)); //Temas de Ayuda
        for (int i=0; i<ayuda.length; i++) {
            ayuda[i]=new MenuItem(idioma.texto(9111+i));
            ayudas.add(ayuda[i]);
        }
        menuAyuda.add(ayudas);
        menuAyuda.addSeparator(); //----------
        licencia=new MenuItem(idioma.texto(98)); //Licencia
        menuAyuda.add(licencia);
        acercade=new MenuItem(idioma.texto(99)); //Acerca de...
        menuAyuda.add(acercade);

        barra.setHelpMenu(menuAyuda);
        barra.add(menuAyuda);
        setMenuBar(barra);

        //Realiza Llamada-NuevaLlamada
        nueva.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                if (!solicitaLlamada) dialogo(11);
            }
        });

        //Realiza Llamada-Colgar
        colgar.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                solicitaColgar=true;
            }
        });

        //Realiza Llamada-Salir
        salir.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                fin=true;
            }
        });

        //Realiza Configuracion-verSIP
        setSIP("No debugging information about SIP..."); //Informacion SIP
        textoSIP=new JTextArea(getSIP());
        ventanaTexto=new JScrollPane(textoSIP);
        getContentPane().add(ventanaTexto);

        verSIP.addItemListener(new ItemListener() {
            public void itemStateChanged(ItemEvent e) {
                refresh();
            }
        });

        //Realiza Configuracion-infoTX
        infoTX.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                solicitaTX=true;
            }
        });

        //Realiza Configuracion-Preferencias
        preferencias.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                solicitaPref=true;
            }
        });

        //Realiza Ayuda-Temas de Ayuda
        for (int i=0; i<ayuda.length; i++) {
            indexAyuda=i;
            ayuda[i].addActionListener(new ActionListener() {
                int miIndex=indexAyuda; //Cada objeto captura su tema de ayuda

                public void actionPerformed(ActionEvent e) {
                    indexAyuda=miIndex;
                    dialogo(91);
                }
            });
        }

        //Realiza Ayuda-Licencia
        licencia.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogo(98);
            }
        });

        //Realiza Ayuda-AcercaDe
        acercade.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                dialogo(99);
            }
        });

        //Centrar la ventana
        setVisible(true);
        width=getInsets().left+width+getInsets().right;
        height=getInsets().top+height+getInsets().bottom;
        if (height>screenSize.height) height=screenSize.height;
        if (width>screenSize.width) width=screenSize.width;

        setSize(width,height);
        setLocation((screenSize.width - width)/2,
                    (screenSize.height - height)/2);

        setIconImage((new ImageIcon("img"+java.io.File.separator+"icono.png"))
                     .getImage());

        setTitle(titulo);

        refresh();
        verSIP.setState(false);
    }

    public void paint(Graphics g) {
        ventanaTexto.setVisible(verSIP.getState());
        if ( (fondo!=null) && !(verSIP.getState()) )
                g.drawImage(fondo,getInsets().left,getInsets().top,this);
    }

    //Metodo encargado de la creacion de los dialogos que necesite el interfaz
    private void dialogo(int n) {

        if (n==11) { //Dialogo de Nueva Llamada

            String respuesta=JOptionPane.showInputDialog(this,idioma.texto(111),
                             idioma.texto(11),JOptionPane.QUESTION_MESSAGE);
            if ((respuesta!=null)&&(respuesta.length()>0)) {
                uriRemoto=respuesta.trim();
                solicitaLlamada=true;
                hayLlamada=true;
            }

        } else if (n==91) { //Dialogo de Temas de Ayuda

            String textoAyuda=idioma.texto(911); //Mensaje si no carga fichero
            File f=new File("lang"+File.separator+idioma.texto(912)
                            +File.separator+(indexAyuda+1)+".txt");

            if (f.canRead()) try {

                RandomAccessFile fichero=new RandomAccessFile(f,"r");
                textoAyuda="";
                while (fichero.getFilePointer()<fichero.length()) {
                    textoAyuda+=fichero.readLine();
                    textoAyuda+=RETURN;
                }
                fichero.close();

            } catch (Exception e) {
                textoAyuda+=RETURN;
                textoAyuda+=idioma.texto(913)+RETURN;
                textoAyuda+=e;
            }

            JTextArea ta=new JTextArea(textoAyuda,20,80);
            ta.setEditable(false);
            Object mensaje[]=new Object[]{idioma.texto(9111+indexAyuda),
                                          new JScrollPane(ta)};
            JOptionPane.showOptionDialog(this,mensaje,
                                         idioma.texto(91),
                                         JOptionPane.DEFAULT_OPTION,
                                         JOptionPane.INFORMATION_MESSAGE,
                                         null,null,null);

        } else if (n==98) { //Dialogo de Licencia

            String license=idioma.texto(981); //Mensaje si no carga el fichero
            File f=new File("licence"+File.separator+"gpl.txt");
            if (f.canRead()) try {

                RandomAccessFile fichero=new RandomAccessFile(f,"r");
                license="";
                while (fichero.getFilePointer()<fichero.length()) {
                    license+=fichero.readLine();
                    license+=RETURN;
                }
                fichero.close();

            } catch (Exception e) {
                license+=RETURN;
                license+=idioma.texto(982)+RETURN;
                license+=e;
            }

            JTextArea ta=new JTextArea(license,20,80);
            ta.setEditable(false);
            Object mensaje[]=new Object[]{idioma.texto(983),
                                          new JScrollPane(ta)};
            JOptionPane.showOptionDialog(this,mensaje,
                                         idioma.texto(984),
                                         JOptionPane.DEFAULT_OPTION,
                                         JOptionPane.INFORMATION_MESSAGE,
                                         null,null,null);

        } else if (n==99) { //Dialogo de Acerca de

            JOptionPane.showMessageDialog(this,idioma.texto(991)+
                    (VERSION/100)+"."+((VERSION%100)<10?"0":"")+(VERSION%100)+
                    idioma.texto(992)+RETURN+RETURN+
                    idioma.texto(993)+RETURN+
                    idioma.texto(994)+RETURN+
                    idioma.texto(995)+RETURN+
                    idioma.texto(996)+RETURN+RETURN+
                    idioma.texto(997)+RETURN+
                    idioma.texto(998)+RETURN+
                    idioma.texto(999)+RETURN,
                    idioma.texto(99),
                    JOptionPane.INFORMATION_MESSAGE);

        } else if (n==100) { //Dialogo cuando JMF no encontrado

            JOptionPane.showMessageDialog(this,idioma.texto(101)+RETURN+
                                          idioma.texto(102)+RETURN+RETURN+
                                          idioma.texto(103)+RETURN+
                                          idioma.texto(104)+RETURN,
                                          idioma.texto(100),
                                          JOptionPane.INFORMATION_MESSAGE);
        }

    }

    /**
     * Metodo para avisar de llamada entrante
     * @param remoto usuario que solicita la llamada
     * @return int
     */
    public int llamada(String remoto) {
        int respuesta;

        if (!hayJMF) respuesta=JOptionPane.NO_OPTION;
        else if (autoresponder.getState()) respuesta=JOptionPane.YES_OPTION;
        else respuesta=JOptionPane.showOptionDialog(null,
                       idioma.texto(112)+remoto+idioma.texto(113),
                       idioma.texto(114),
                       JOptionPane.YES_NO_OPTION,
                       JOptionPane.QUESTION_MESSAGE,
                       null,null,null);

        if (respuesta==JOptionPane.YES_OPTION) hayLlamada=true;
        return respuesta;
    }

    /**
     * Metodo para cerrar la conferencia en curso
     */
    public void colgar() {
        hayLlamada=false;
        solicitaColgar=false;
    }

    /**
     * Metodo para indicar que se ha aceptado la llamada solicitada
     */
    public void aceptar() {
        if (solicitaLlamada==true) {
            hayLlamada=true;
            solicitaLlamada=false;
        }
    }

    /**
     * Metodo para indicar que se ha rechazado la llamada solicitada
     */
    public void rechazar() {
        if (solicitaLlamada==true) {
            colgar();
            solicitaLlamada=false;
        }
    }

    /**
     * Metodo que realiza los cambios necesarios para poder funcionar sin JMF
     */
    public void noHayJMF() {
        hayJMF=false;
        nueva.setEnabled(false);
        infoTX.setEnabled(false);
        autoresponder.setEnabled(false);
        preferencias.setEnabled(false);
        dialogo(100);
    }

    //Metodo encargado del cambio de idioma del interfaz
    private void cambioIdioma() {

        menuLlamada.setLabel(idioma.texto(1)); //Llamada
        nueva.setLabel(idioma.texto(11)); //NuevaLlamada
        colgar.setLabel(idioma.texto(12)); //Colgar
        salir.setLabel(idioma.texto(19)); //Salir

        menuConf.setLabel(idioma.texto(2)); //Configuracion
        verSIP.setLabel(idioma.texto(21)); //Ver SIP
        infoTX.setLabel(idioma.texto(22)); //infoTX
        autoresponder.setLabel(idioma.texto(23)); //Autoresponder
        preferencias.setLabel(idioma.texto(29)); //Preferencias

        menuIdioma.setLabel(idioma.texto(3)); //Idioma

        menuAyuda.setLabel(idioma.texto(9)); //Ayuda
        ayudas.setLabel(idioma.texto(91)); //Temas de Ayuda
        for (int i=0; i<ayuda.length; i++) {
            ayuda[i].setLabel(idioma.texto(9111+i));
        }
        licencia.setLabel(idioma.texto(98)); //Licencia
        acercade.setLabel(idioma.texto(99)); //Acerca de...

        refresh();
    }

    /**
     * Metodo que refresca el interfaz
     */
    public void refresh() {
        nueva.setEnabled(hayJMF && !hayLlamada);
        colgar.setEnabled(hayLlamada);
        preferencias.setEnabled(hayJMF && !hayLlamada);
        infoTX.setEnabled(hayJMF);
        autoresponder.setEnabled(hayJMF);
        textoSIP.setText(getSIP());
        repaint();
    }

    /**
    * Metodo que establece la informacion de depuracion sobre el protocolo SIP
    * @param string Cadena de texto con la informacion de depuracion sobre SIP
    */
    public synchronized void setSIP(String string) {
        sipDebug=new String(string);
    }

    /**
     * Metodo que obtiene la informacion de depuracion sobre el protocolo SIP
     * @return String Cadena de texto con la informacion de depuracion sobre SIP
     */
    public synchronized String getSIP() {
        return sipDebug;
    }

    //Metodo para cargar la imagen de fondo
    private void cargarImagen() {
        MediaTracker tracker=new MediaTracker(this);

        tracker.addImage(fondo,0); //Controlamos la carga de la imagen de fondo
        try {tracker.waitForAll();}
        catch (InterruptedException e) {System.err.println("Error with image");}
        if (tracker.isErrorAny()) fondo=null;
        tracker.removeImage(fondo);
    }

    /**
     * Destructor del Interfaz.
     * Se libera la memoria ocupada y se cierra el Interfaz.
     */
    public void salir() {
        idioma.salir();
        dispose();
    }

}
