/*
*   umasip is a Videoconference program that uses the SIP protocol (RFC 3261)
*   Copyright (C) 2004 Pablo Montoro Escao
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import javax.media.*; //Java Media Framework API (JMF)
import javax.media.rtp.*; //Java Media Framework API (JMF)
import javax.media.rtp.event.*; //Java Media Framework API (JMF)

/**
 * <p>Programa: umasip</p>
 * <p>Copyright: Copyright GNU/GPL (c) 2004 Pablo Montoro Escao</p>
 * @author Pablo Montoro Escao
 * @version 1.00
 * @since SDK 1.4.2.
 */

public class Llamada extends Frame {

    final String TITULO=new String("Call Frame"); //Titulo por defecto
    final String BOTON=new String("Hang"); //Texto del boton por defecto
    final Dimension DIMENSION=new Dimension(400,400); //Dimension por defecto
    final String FONDO=("img"+java.io.File.separator+"fondo.png");
    final int ALTO_BOTON=20; //Altura del boton

    private int ancho,alto; //Dimensiones delcontenido de la ventana
    private Image img; //Imagen a mostrar en la ventana de llamada
    private Insets insets=getInsets();
    private Button boton=new Button(BOTON);

    public boolean solicitaColgar=false; //Indica si se ha solicitado colgar
    private Processor processorAudio=null;
    private Processor processorVideo=null;

    private RTPManager[] managerTX,managerRX;
    private Object monitor=new Object();
    private Object monitor2=new Object();
    private long espera=4000; //Tiempo maximo de espera (en milisegundos)
    private boolean fallo=false;//Usado para indicar fallo de un processor
    private boolean dataReceived=false; //Indica si se ha recibido algo
    private Component vc=null;

    /**
     * Constructor de Llamada. Se toman dimensiones por defecto.
     */
    public Llamada() {
        setImagen(FONDO);
        int width=(img!=null)?img.getWidth(this):DIMENSION.width;
        int height=(img!=null)?img.getHeight(this):DIMENSION.height;
        init(TITULO,BOTON,width,height);
    }

    /**
     * Constructor de Llamada. Se toman dimensiones por defecto.
     * @param titulo Nombre que aparecera en el Interfaz
     */
    public Llamada(String titulo) {
        setImagen(FONDO);
        int width=(img!=null)?img.getWidth(this):DIMENSION.width;
        int height=(img!=null)?img.getHeight(this):DIMENSION.height;
        init(titulo,BOTON,width,height);
    }

    /**
     * Constructor de Llamada. Se toman dimensiones por defecto.
     * @param titulo Nombre que aparecera en el Interfaz
     * @param botonText Texto del boton utilizado para colgar
     */
    public Llamada(String titulo,String botonText) {
        setImagen(FONDO);
        int width=(img!=null)?img.getWidth(this):DIMENSION.width;
        int height=(img!=null)?img.getHeight(this):DIMENSION.height;
        init(titulo,botonText,width,height);
    }

    /**
     * Constructor de Llamada
     * @param titulo Nombre que aparecera en el Interfaz
     * @param dimension Dimensiones del Interfaz
     */
    public Llamada(String titulo,Dimension dimension) {
        setImagen(FONDO);
        init(titulo,BOTON,dimension.width,dimension.height);
    }

    /**
     * Constructor de Llamada
     * @param titulo Nombre que aparecera en el Interfaz
     * @param dimension Dimensiones del Interfaz
     * @param botonText Texto del boton utilizado para colgar
     */
    public Llamada(String titulo,String botonText,Dimension dimension) {
        setImagen(FONDO);
        init(titulo,botonText,dimension.width,dimension.height);
    }

    /**
     * Constructor de Llamada
     * @param titulo Nombre que aparecera en el Interfaz
     * @param width Ancho del Interfaz
     * @param height Alto del Interfaz
     */
    public Llamada(String titulo,int width,int height) {
        setImagen(FONDO);
        init(titulo,BOTON,width,height);
    }

    /**
     * Constructor de Llamada
     * @param titulo Nombre que aparecera en el Interfaz
     * @param width Ancho del Interfaz
     * @param height Alto del Interfaz
     * @param botonText Texto del boton utilizado para colgar
     */
    public Llamada(String titulo,String botonText,int width,int height) {
        setImagen(FONDO);
        init(titulo,botonText,width,height);
    }

    //Metodo constructor general
    private void init(String titulo,String botonText,int width,int height) {

        //Centrar la ventana
        Dimension screenSize=Toolkit.getDefaultToolkit().getScreenSize();
        if (height>screenSize.height) {
            height=screenSize.height;
        }
        if (width>screenSize.width) {
            width=screenSize.width;
        }

        setSize(width,height);
        setLocation((screenSize.width-width)/2,
                    (screenSize.height-height)/2);
        setIconImage((new ImageIcon("img"+java.io.File.separator+"icono.png"))
                     .getImage());
        setTitle(titulo);
        setVisible(true);
        setLayout(null);
        setResizable(false);
        insets=getInsets();

        boton.setLabel(botonText);
        boton.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                solicitaColgar=true;
            }
        });
        add(boton);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                solicitaColgar=true;
            }
            public void windowClosed(WindowEvent e) {
                solicitaColgar=true;
            }
        });

        addWindowStateListener(new WindowAdapter() {
            public void windowStateChanged(WindowEvent e) {
                repaint();
            }
        });
    }

    public void paint(Graphics g) {

        if (vc!=null) {
            ancho=vc.getWidth();
            alto=vc.getHeight();
        } else if (img!=null) {
            ancho=img.getWidth(this);
            alto=img.getHeight(this);
        } else {
            ancho=DIMENSION.width;
            alto=DIMENSION.height;
        }
        if (ancho<100) ancho=100; //Dimensiones minimas para la imagen
        if (alto<100) alto=100;

        setSize(insets.left+ancho+insets.right,
                insets.top+alto+ALTO_BOTON+insets.bottom);

        if (vc!=null) vc.setLocation(insets.left,insets.top);
        else if (img!=null) g.drawImage(img,insets.left,insets.top,this);

        boton.setBounds(insets.left,insets.top+alto,ancho,ALTO_BOTON);
    }

    /**
     * Establece la imagen a ver en la ventana de Llamada
     * @param imagen Nombre del fichero con la imagen a cargar
     */
    public void setImagen(String imagen) {
        MediaTracker tracker=new MediaTracker(this);

        img=getToolkit().getImage(imagen);

        tracker.addImage(img,0); //Controlamos la carga de la imagen
        try {tracker.waitForAll();}
        catch (InterruptedException e) {System.err.println("Error with image");}
        if (tracker.isErrorAny()) img=null;
        tracker.removeImage(img);

        repaint();
    }

    /**
     * Metodo de transmision de los datos multimedia
     * @param audio Localizador del medio a transmitir
     * @param video Localizador del medio a transmitir
     * @param ipLocal Direccion IP local
     * @param ipRemoto Direccion IP remota
     * @param puertoAudio Puerto de audio en la direccion IP remota
     * @param puertoVideo Puerto de video en la direccion IP remota
     */
    public void TX(String ipLocal,String ipRemoto,Processor audio,
                   int puertoAudio,Processor video,int puertoVideo) {

        int i=0;
        processorAudio=audio;
        processorVideo=video;

        if ((processorAudio==null)&&(processorVideo==null)) {
            System.err.println("Transmitting nothing (No Media Locator)");
        } else {
            if (!realizar(processorAudio)) processorAudio=null;
            else {
                i++;
                processorAudio.start();
            }
            if (!realizar(processorVideo)) processorVideo=null;
            else {
                i++;
                processorVideo.start();
            }
        }

        managerTX=new RTPManager[i];

        i=0;
        if (processorAudio!=null) {
            try {
                managerTX[i]=RTPManager.newInstance();
                managerTX[i].initialize(new SessionAddress(InetAddress
                                        .getByName(ipLocal),puertoAudio));
                managerTX[i].addTarget(new SessionAddress(InetAddress
                                       .getByName(ipRemoto),puertoAudio));
                managerTX[i].createSendStream(processorAudio.getDataOutput(),0)
                            .start();

            } catch (Exception  e) {
                System.err.println("Error audio: "+e);
                if (processorAudio!=null) {
                    processorAudio.close();
                    processorAudio=null;
                }
            }
            i++;
        }
        if (processorVideo!=null) {
            try {
                managerTX[i]=RTPManager.newInstance();
                managerTX[i].initialize(new SessionAddress(InetAddress
                                        .getByName(ipLocal),puertoVideo));
                managerTX[i].addTarget(new SessionAddress(InetAddress
                                       .getByName(ipRemoto),puertoVideo));
                managerTX[i].createSendStream(processorVideo.getDataOutput(),0)
                            .start();

            } catch (Exception  e) {
                System.err.println("Error video: "+e);
                if (processorVideo!=null) {
                    processorVideo.close();
                    processorVideo=null;
                }
            }
        }

    }

    //Realizar el processor
    private synchronized boolean realizar(Processor proc) {
        if (proc==null) return false;

        proc.addControllerListener(new ControllerListener() {

            public void controllerUpdate(ControllerEvent ce) {
                if (ce instanceof ControllerClosedEvent) {
                    synchronized (monitor) {
                        fallo=true;
                        monitor.notifyAll();
                    }
                }

                if (ce instanceof ControllerEvent) {
                    synchronized (monitor) {
                        monitor.notifyAll();
                    }
                }
            }
        });

        fallo=false;
        proc.realize();

        //Esperamos que el processor alcance el estado Realized... o de fallo
        while (!solicitaColgar &&
               proc.getState()<Processor.Realized && !fallo) {
            synchronized (monitor) {
                try {
                    monitor.wait(500);
                } catch (InterruptedException ie) {
                    return false;
                }
            }
        }

        return !fallo;
    }

    /**
     * Metodo de recepcion de los datos multimedia
     * @param ipLocal Direccion IP local
     * @param ipRemoto Direccion IP remota
     * @param puertoAudio Puerto en el que recibir los datos multimedia de audio
     * @param puertoVideo Puerto en el que recibir los datos multimedia de video
     * @return Devuelve si se han recibido datos correctamente
     */
    public boolean RX(String ipLocal,String ipRemoto,
                      int puertoAudio,int puertoVideo) {
        int i=0;
        dataReceived=false;

        if ((puertoAudio==0) && (puertoVideo==0)) return true;

        if (puertoAudio!=0) i++;
        if (puertoVideo!=0) i++;
        managerRX=new RTPManager[i];

        i=0;
        try {

            if (puertoAudio!=0) {

                managerRX[i]=RTPManager.newInstance();
                managerRX[i].addReceiveStreamListener(new rsl());
                managerRX[i].initialize(new SessionAddress(
                             InetAddress.getByName(ipLocal),puertoAudio));
                managerRX[i].addTarget(new SessionAddress(
                             InetAddress.getByName(ipRemoto),puertoAudio));
                i++;
            }
            if (puertoVideo!=0) {

                managerRX[i]=RTPManager.newInstance();
                managerRX[i].addReceiveStreamListener(new rsl());
                managerRX[i].initialize(new SessionAddress(
                             InetAddress.getByName(ipLocal),puertoVideo));
                managerRX[i].addTarget(new SessionAddress(
                             InetAddress.getByName(ipRemoto),puertoVideo));
            }

        } catch (Exception e){
            System.err.println("Error creating RTP session for "+
                               ipRemoto+": "+e.getMessage());
            return false;
        }

        //Esperamos la recepcion de datos
        long t1=System.currentTimeMillis(); //Tomamos una muestra temporal

        if ((puertoAudio!=0) || (puertoVideo!=0)) try{
            synchronized (monitor2) {
                while (!solicitaColgar && !dataReceived
                       && (System.currentTimeMillis()-t1)<espera) {

                    System.err.println("Waiting for RTP data");
                    monitor2.wait(500);
                }
            }
        } catch (Exception e) {}

        if (!dataReceived) {
            System.err.println("No RTP data received");
            for (int j=0; j<managerRX.length; j++) {
                if (managerRX[j]!=null) {
                    managerRX[j].removeTargets("Session ended");
                    managerRX[j].dispose();
                    managerRX[j]=null;
                }
            }
            return false;
        }

        return true;
    }

    //Clase interna para implementar un Listener para la recepcion de flujos
    private class rsl implements ReceiveStreamListener {
        public synchronized void update(ReceiveStreamEvent evt) {

            ReceiveStream rs=evt.getReceiveStream();

            if (evt instanceof RemotePayloadChangeEvent) {
                System.err.println("Received an RTP PayloadChangeEvent. "
                                   +"Not yet supported.");

            } else if (evt instanceof NewReceiveStreamEvent) {
                try {

                    Player p=javax.media.Manager
                             .createPlayer(rs.getDataSource());
                    if (p==null) return;

                    p.addControllerListener(new cl());
                    p.realize();

                    synchronized (monitor2) {
                        dataReceived=true;
                        monitor2.notifyAll();
                    }

                } catch (Exception e) {
                    System.err.println(e);
                    return;
                }

            }

        }
    }

    //Metdodo que coloca la imagen de video en la ventana
    private void addVideo(Player p) {
        setLayout(new BorderLayout());
        if ((vc=p.getVisualComponent())!=null) add(vc);
        vc.setSize(vc.getPreferredSize());
        setSize(insets.left+vc.getPreferredSize().width+insets.right,
                insets.top+vc.getPreferredSize().height+insets.bottom);
        setVisible(true);
    }

    //Clase interna para implementar el controlador del processor de RX
    private class cl implements ControllerListener {
        public synchronized void controllerUpdate(ControllerEvent ce) {

            Player p=(Player)ce.getSourceController();
            if (p==null) return;

            //Obtenemos la imagen cuando esta lista para ser presentada
            if (ce instanceof RealizeCompleteEvent) {
                if ((p.getVisualComponent())!=null) addVideo(p);
                p.start();
            }

            if (ce instanceof ControllerErrorEvent) {
                System.err.println("Controller Error in Receiver: "+ce);
                p.removeControllerListener(this);
                p.close();
            }
        }
    }

    /**
     * Destructor de la Llamada.
     * Se libera la memoria ocupada y se cierra la Ventana.
     */
    public void salir() {

        //Fin de RX
        if (managerRX!=null) for (int i=0; i<managerRX.length; i++) {
            if (managerRX[i]!=null) {
                managerRX[i].removeTargets("Closing session");
                managerRX[i].dispose();
                managerRX[i]=null;
            }
        }

        //Fin de TX
        if (managerTX!=null) for (int i=0; i<managerTX.length; i++) {
            if (managerTX[i]!=null) {
                managerTX[i].removeTargets("Closing session");
                managerTX[i].dispose();
                managerTX[i]=null;
            }
        }

        try {Thread.sleep(500);} catch(Exception e) {}
        setVisible(false);
        dispose();
    }

}
