/*
*   umasip is a Videoconference program that uses the SIP protocol (RFC 3261)
*   Copyright (C) 2004 Pablo Montoro Escao
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.net.*;
import javax.media.*; //Java Media Framework API (JMF)
import javax.media.format.*; //Java Media Framework API (JMF)
import javax.media.protocol.*; //Java Media Framework API (JMF)
import javax.media.control.*; //Java Media Framework API (JMF)

/**
 * <p>Programa: umasip</p>
 * <p>Copyright: Copyright GNU/GPL (c) 2004 Pablo Montoro Escao</p>
 * @author Pablo Montoro Escao
 * @version 1.00
 * @since SDK 1.4.2.
 */

public class Preferencias extends JFrame {

    private final Dimension DIMENSION=new Dimension(600,400);//Dimension defecto
    private final String RETURN=new String("\n"); //Cadena para fin de linea
    private final int ALTO_BOTON=30; //Altura del boton
    private final int ANCHO_BOTON=(int) (DIMENSION.getWidth()/5)-20; //Ancho

    private Idioma idioma=new Idioma();
    public boolean solicitaCambio=false;
    public String usuario,ip;
    public Processor audio=null; //Processor encargado del audio para TX
    public Processor video=null; //Processor encargado del video para TX
    private Processor audioTemp,videoTemp; //Processor temporales
    private MediaLocator audioML=null;
    private MediaLocator videoML=null;

    private Insets insets=getInsets(); //Bordes de la ventana
    private int ancho,alto; //Dimensiones del area de dibujo

    private TextField campoUsuario,campoIP; //Componentes de la ventana
    private Button boton1,boton2,boton3,boton4;
    private Label txt1,txt2,txt3,txt4,txt5,txt6;
    private Choice dispAudio,dispVideo;
    private java.util.Vector listaAudio,listaVideo; //Listas de dispositivos
    private int indexAudio,indexVideo; //Indice del dispositivo seleccionado
    private Choice formatoAudio,formatoVideo;
    private int indexFormatoAudio,indexFormatoVideo; //Indices Formatos
    private boolean hayFormatoAudio,hayFormatoVideo;


    /**
     * Constructor de Preferencias. Se toman dimensiones por defecto.
     */
    public Preferencias() {
        init(DIMENSION.width,DIMENSION.height);
    }

    /**
     * Constructor de Preferencias. Se toman dimensiones por defecto.
     * @param i Modulo de idioma a emplear
     */
    public Preferencias(Idioma i) {
        idioma=i;
        init(DIMENSION.width,DIMENSION.height);
    }

    /**
     * Constructor de Preferencias
     * @param dimension Dimensiones del Interfaz
     */
    public Preferencias(Dimension dimension) {
        init(dimension.width,dimension.height);
    }

    /**
     * Constructor de Preferencias
     * @param i Modulo de idioma a emplear
     * @param dimension Dimensiones del Interfaz
     */
    public Preferencias(Idioma i,Dimension dimension) {
        idioma=i;
        init(dimension.width,dimension.height);
    }

    /**
     * Constructor de Preferencias
     * @param width Ancho del Interfaz
     * @param height Alto del Interfaz
     */
    public Preferencias(int width,int height) {
        init(width,height);
    }

    /**
     * Constructor de Preferencias
     * @param i Modulo de idioma a emplear
     * @param width Ancho del Interfaz
     * @param height Alto del Interfaz
     */
    public Preferencias(Idioma i,int width,int height) {
        idioma=i;
        init(width,height);
    }

    //Metodo constructor general
    private void init(int width,int height) {

        //Centrar la ventana
        Dimension screenSize=Toolkit.getDefaultToolkit().getScreenSize();
        if (height>screenSize.height) height=screenSize.height;
        if (width>screenSize.width) width=screenSize.width;

        setSize(width,height);
        setLocation((screenSize.width-width)/2,(screenSize.height-height)/2);
        insets=getInsets();
        ancho=width-insets.left-insets.right;
        alto=height-insets.top-insets.bottom;

        setIconImage((new ImageIcon("img"+java.io.File.separator+"icono.png"))
                     .getImage());
        setTitle(idioma.texto(29));
        setVisible(false);
        getContentPane().setLayout(null);

        //Elementos que apareceran en la ventana
        txt1=new Label(idioma.texto(2911)+": ",Label.RIGHT);
        txt2=new Label(idioma.texto(2912)+": ",Label.RIGHT);
        txt3=new Label(idioma.texto(2913)+": ",Label.RIGHT);
        txt4=new Label(idioma.texto(2914)+": ",Label.RIGHT);
        txt5=new Label(idioma.texto(2915)+": ",Label.RIGHT);
        txt6=new Label(idioma.texto(2916)+": ",Label.RIGHT);
        getContentPane().add(txt1);
        getContentPane().add(txt2);
        getContentPane().add(txt3);
        getContentPane().add(txt4);
        getContentPane().add(txt5);
        getContentPane().add(txt6);

        usuario="anonymous";
        campoUsuario=new TextField(usuario,Label.LEFT);
        getContentPane().add(campoUsuario);

        try {
            ip=InetAddress.getLocalHost().getHostAddress().toString();
        } catch (UnknownHostException e) {
            ip="127.0.0.1";
        }
        campoIP=new TextField(ip,Label.LEFT);
        getContentPane().add(campoIP);

        boton1=new Button(idioma.texto(2917)); //Nombre por defecto
        boton1.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                campoUsuario.setText("anonymous");
            }
        });
        getContentPane().add(boton1);

        boton2=new Button(idioma.texto(2918)); //Autodetectar direccion IP
        boton2.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                try {
                    campoIP.setText(InetAddress.getLocalHost()
                                    .getHostAddress().toString());
                } catch (UnknownHostException e2) {
                    campoIP.setText("127.0.0.1");
                }
            }
        });
        getContentPane().add(boton2);

        dispAudio=new Choice(); //Eleccion dispositivo Audio
        listaAudio=CaptureDeviceManager
                    .getDeviceList(new AudioFormat(AudioFormat.LINEAR));
        for (int i=0; i<listaAudio.size(); i++) {
            dispAudio.add(((CaptureDeviceInfo)listaAudio.elementAt(i))
                          .getName());
        }
        dispAudio.add(idioma.texto(2921));
        dispAudio.addItemListener(new audioListener());
        getContentPane().add(dispAudio);

        formatoAudio=new Choice(); //Eleccion del formato de Audio a transmitir
        indexFormatoAudio=0;
        hayFormatoAudio=true;
        getContentPane().add(formatoAudio);

        dispVideo=new Choice(); //Eleccion dispositivo Video
        listaVideo=CaptureDeviceManager
                    .getDeviceList(new VideoFormat(VideoFormat.RGB));
        for (int i=0; i<listaVideo.size(); i++) {
            dispVideo.add(((CaptureDeviceInfo)listaVideo.elementAt(i))
                          .getName());
        }
        dispVideo.add(idioma.texto(2922));
        dispVideo.addItemListener(new videoListener());
        getContentPane().add(dispVideo);

        formatoVideo=new Choice(); //Eleccion del formato de Video a transmitir
        indexFormatoVideo=0;
        hayFormatoVideo=true;
        getContentPane().add(formatoVideo);

        //Inicializacion de atributos de preferencias
        usuario=campoUsuario.getText();
        ip=campoIP.getText();
        indexAudio=0;
        indexVideo=0;
        if (indexAudio==listaAudio.size()) audioML=null;
        else audioML=((CaptureDeviceInfo)listaAudio.elementAt(indexAudio))
                   .getLocator();
        if (audioML!=null) audioTemp=crearProcessor(audioML,formatoAudio,
                                                    indexFormatoAudio);
        if (audioTemp==null) {
            indexFormatoAudio=-1;
            formatoAudio.add(idioma.texto(2923));
        }
        hayFormatoAudio=(indexFormatoAudio!=-1);
        matarProcessor(audioTemp);

        if (indexVideo==listaVideo.size()) videoML=null;
        else videoML=((CaptureDeviceInfo)listaVideo.elementAt(indexVideo))
                   .getLocator();
        if (videoML!=null) videoTemp=crearProcessor(videoML,formatoVideo,
                                                    indexFormatoVideo);
        if (videoTemp==null) {
           indexFormatoVideo=-1;
           formatoVideo.add(idioma.texto(2923));
        }
        hayFormatoVideo=(indexFormatoVideo!=-1);
        matarProcessor(videoTemp);

        aceptar(); //Aceptamos los valores adoptados por la inicializacion

        //Botones de Aceptar o Cancelar los cambios
        boton3=new Button(idioma.texto(2919)); //Aceptar
        boton3.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                aceptar();
                solicitaCambio=true;
                setVisible(false);
            }
        });
        getContentPane().add(boton3);

        boton4=new Button(idioma.texto(2920)); //Cancelar
        boton4.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                cancelar();
                setVisible(false);
            }
        });
        getContentPane().add(boton4);

        addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent e) {
                cancelar();
                setVisible(false);
            }
            public void windowClosed(WindowEvent e) {
                cancelar();
                setVisible(false);
            }
        });

        addWindowStateListener(new WindowAdapter() {
            public void windowStateChanged(WindowEvent e) {
                repaint();
            }
        });
    }

    //Acciones para el boton de Aceptar
    private void aceptar() {
        usuario=campoUsuario.getText();

        ip=campoIP.getText();

        indexAudio=dispAudio.getSelectedIndex();
        if (indexAudio==listaAudio.size()) audioML=null;
        else audioML=((CaptureDeviceInfo)listaAudio
                      .elementAt(indexAudio)).getLocator();

        if (hayFormatoAudio)
            indexFormatoAudio=formatoAudio.getSelectedIndex();
        else
            indexFormatoAudio=-1;
        matarProcessor(audio);
        audio=crearProcessor(audioML,formatoAudio,indexFormatoAudio);
        if (hayFormatoAudio) formatoAudio.select(indexFormatoAudio);
        else formatoAudio.add(idioma.texto(2923));

        indexVideo=dispVideo.getSelectedIndex();
        if (indexVideo==listaVideo.size()) videoML=null;
        else videoML=((CaptureDeviceInfo)listaVideo
                      .elementAt(indexVideo)).getLocator();

        if (hayFormatoVideo)
            indexFormatoVideo=formatoVideo.getSelectedIndex();
        else
            indexFormatoVideo=-1;
        matarProcessor(video);
        video=crearProcessor(videoML,formatoVideo,indexFormatoVideo);
        if (hayFormatoVideo) formatoVideo.select(indexFormatoVideo);
        else formatoVideo.add(idioma.texto(2923));
    }

    //Acciones para el boton de Cancelar
    private void cancelar() {
        campoUsuario.setText(usuario);
        campoIP.setText(ip);
        dispAudio.select(indexAudio);
        dispVideo.select(indexVideo);

        matarProcessor(audio);
        audio=crearProcessor(audioML,formatoAudio,indexFormatoAudio);
        if (indexFormatoAudio<0) formatoAudio.add(idioma.texto(2923));
        else formatoAudio.select(indexFormatoAudio);
        hayFormatoAudio=(indexFormatoAudio!=-1);

        matarProcessor(video);
        video=crearProcessor(videoML,formatoVideo,indexFormatoVideo);
        if (indexFormatoVideo<0) formatoVideo.add(idioma.texto(2923));
        else formatoVideo.select(indexFormatoVideo);
        hayFormatoVideo=(indexFormatoVideo!=-1);
    }

    public void paint(Graphics g) {

        ancho=getSize().width-insets.left-insets.right;
        alto=getSize().height-insets.top-insets.bottom;

        setTitle(idioma.texto(29));
        txt1.setText(idioma.texto(2911)+": ");
        txt2.setText(idioma.texto(2912)+": ");
        txt3.setText(idioma.texto(2913)+": ");
        txt4.setText(idioma.texto(2914)+": ");
        txt5.setText(idioma.texto(2915)+": ");
        txt6.setText(idioma.texto(2916)+": ");
        boton1.setLabel(idioma.texto(2917));
        boton2.setLabel(idioma.texto(2918));
        boton3.setLabel(idioma.texto(2919));
        boton4.setLabel(idioma.texto(2920));

        txt1.setBounds(insets.left,insets.top+1*alto/10,3*ancho/10,ALTO_BOTON);
        campoUsuario.setBounds(insets.left+3*ancho/10, insets.top+1*alto/10,
                               3*ancho/10+ANCHO_BOTON, ALTO_BOTON);
        boton1.setBounds(insets.left+8*ancho/10, insets.top+1*alto/10,
                         ANCHO_BOTON, ALTO_BOTON);

        txt2.setBounds(insets.left,insets.top+2*alto/10,3*ancho/10,ALTO_BOTON);
        campoIP.setBounds(insets.left+3*ancho/10, insets.top+2*alto/10,
                          3*ancho/10+ANCHO_BOTON, ALTO_BOTON);
        boton2.setBounds(insets.left+8*ancho/10, insets.top+2*alto/10,
                         ANCHO_BOTON, ALTO_BOTON);

        txt3.setBounds(insets.left,insets.top+3*alto/10,3*ancho/10,ALTO_BOTON);
        dispAudio.setBounds(insets.left+3*ancho/10, insets.top+3*alto/10,
                            5*ancho/10+ANCHO_BOTON, ALTO_BOTON);

        txt4.setBounds(insets.left,insets.top+4*alto/10,3*ancho/10,ALTO_BOTON);
        formatoAudio.setBounds(insets.left+3*ancho/10, insets.top+4*alto/10,
                            5*ancho/10+ANCHO_BOTON, ALTO_BOTON);

        txt5.setBounds(insets.left,insets.top+5*alto/10,3*ancho/10,ALTO_BOTON);
        dispVideo.setBounds(insets.left+3*ancho/10, insets.top+5*alto/10,
                           5*ancho/10+ANCHO_BOTON, ALTO_BOTON);

        txt6.setBounds(insets.left,insets.top+6*alto/10,3*ancho/10,ALTO_BOTON);
        formatoVideo.setBounds(insets.left+3*ancho/10, insets.top+6*alto/10,
                            5*ancho/10+ANCHO_BOTON, ALTO_BOTON);

        boton3.setBounds(insets.left+6*ancho/10, insets.top+8*alto/10,
                         ANCHO_BOTON, ALTO_BOTON);
        boton4.setBounds(insets.left+8*ancho/10, insets.top+8*alto/10,
                         ANCHO_BOTON, ALTO_BOTON);
    }

    //Listener para cambio de dispositivo de Audio
    private class audioListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            MediaLocator ml; //Variable temporal para el Locator del dispositivo

            matarProcessor(audioTemp);
            matarProcessor(audio);
            if (dispAudio.getSelectedIndex()==listaAudio.size()) ml=null;
            else ml=((CaptureDeviceInfo)listaAudio
                     .elementAt(dispAudio.getSelectedIndex())).getLocator();
            audioTemp=crearProcessor(ml,formatoAudio,0);
            hayFormatoAudio=(audioTemp!=null);
            if (!hayFormatoAudio) formatoAudio.add(idioma.texto(2923));
            matarProcessor(audioTemp);

            repaint();
        }
    }

    //Listener para cambio de dispositivo de Video
    private class videoListener implements ItemListener {
        public void itemStateChanged(ItemEvent e) {
            MediaLocator ml; //Variable temporal para el Locator del dispositivo

            matarProcessor(videoTemp);
            matarProcessor(video);
            if (dispVideo.getSelectedIndex()==listaVideo.size()) ml=null;
            else ml=((CaptureDeviceInfo)listaVideo
                     .elementAt(dispVideo.getSelectedIndex())).getLocator();
            videoTemp=crearProcessor(ml,formatoVideo,0);
            hayFormatoVideo=(videoTemp!=null);
            if (!hayFormatoVideo) formatoVideo.add(idioma.texto(2923));
            matarProcessor(videoTemp);

            repaint();
        }
    }

    //Metodo que crea el processor para cada medio
    //Ademas actualiza un Choice con los formatos disponibles para RTP
    private Processor crearProcessor(MediaLocator medio,Choice choice,
                                     int indexFormato) {
        Processor processor; //Variable a devolver
        Format[] listaFormato;
        choice.removeAll();

        if (medio==null) return null; //No hay medio a transmitir

        try {
            processor=Manager.createProcessor(Manager.createDataSource(medio));
        } catch (Exception e) {
            System.err.println("Error creating processor: "+e);
            return null;
        }

        //Esperamos que el processor alcance el estado Configured
        if (!configurar(processor)) {
            System.err.println("Processor not configured");
            return null;
        }

        TrackControl[] pista=processor.getTrackControls();
        if ((pista==null) || (pista.length<1)) {
            System.err.println("No track found in processor");
            return null;
        }

        //Establecemos como formatos validos los que sirvan para transmitir RTP
        processor.setContentDescriptor(new ContentDescriptor
                                       (ContentDescriptor.RAW_RTP));

        int index=0; //Contador de los formatos encontrados
        for (int i=0; i<pista.length; i++) {

            if (pista[i].isEnabled()) {

                listaFormato=pista[i].getSupportedFormats();

                //Seleccionamos un formato soportado para para tramsmitir RTP
                if (listaFormato.length==0) {
                   System.err.println("No valid RTP formats for the tracks");
                   return null;
                } else {

                    for (int j=0; j<listaFormato.length; j++) {
                        choice.add(listaFormato[j].toString());
                        if (indexFormato==index) {
                            pista[i].setFormat(listaFormato[j]);
                        }
                        index++;
                    }
                    if (index==0) return null; //Si ningun formato, no processor

                }
            }
        }

        return processor;
    }

    private synchronized boolean configurar(Processor proc) {

        if (proc!=null) proc.configure();

        //Esperamos que el processor alcance el estado Configured o se cierre
        while ((proc!=null) && (proc.getState()<Processor.Configured)) {
            try {Thread.sleep(50);} catch(Exception e) {e.printStackTrace();}
        }
        return (proc!=null);
    }

    //Metodo que mata un processor
    private void matarProcessor(Processor processor) {
        if (processor!=null) {
            processor.stop();
            processor.close();
        }
    }

    /**
     * Devuelve el tipo de carga RTP/AVP, el formato de audio segun RFC1890
     * @return String Payload Type RTP/AVP segun RFC1890
     */
    public String audioPT() {
        AudioFormat af=null;

        if ( (audio==null) || (!hayFormatoAudio) ) return null;
        else {
            for (int i=0; i<audio.getTrackControls().length; i++) {
                if (audio.getTrackControls()[i].isEnabled()) {
                    af=(AudioFormat)(audio.getTrackControls()[i].getFormat());
                }
            }
            if (af==null) af=new AudioFormat(formatoAudio.getSelectedItem());

            if ( (new AudioFormat(AudioFormat.ULAW_RTP)).isSameEncoding(af) ) {
                return "0"; //RFC1890 RTP/AVP 0=PCMU A 8000 1
            } else if ( (new AudioFormat(AudioFormat.DVI_RTP))
                        .isSameEncoding(af) ) {
                if (af.getSampleRate()==8000) return "5";
                //RFC1890 RTP/AVP 5=DVI4 A 8000 1
                else if (af.getSampleRate()==16000) return "6";
                //RFC1890 RTP/AVP 6=DVI4 A 16000 1
                else if (af.getSampleRate()==11025) return "16";
                //RFC3551 RTP/AVP 16=DVI4 A 11,025 1
                else if (af.getSampleRate()==22050) return "17";
                //RFC3551 RTP/AVP 17=DVI4 A 22,050 1
                else return "-1"; //Formato desconocido
            } else if ( (new AudioFormat(AudioFormat.G723_RTP))
                        .isSameEncoding(af) ) {
                return "4"; //RFC3551 RTP/AVP 4=G723 A 8,000 1
            } else if ( (new AudioFormat(AudioFormat.G728_RTP))
                        .isSameEncoding(af) ) {
                return "15"; //RFC1890 RTP/AVP 15=G728 A 8000 1
            } else if ( (new AudioFormat(AudioFormat.G729_RTP))
                        .isSameEncoding(af) ) {
                return "18"; //RFC3551 RTP/AVP 18=G729 A 8,000 1
            } else if ( (new AudioFormat(AudioFormat.G729A_RTP))
                        .isSameEncoding(af) ) {
                return "18"; //RFC3551 RTP/AVP 18=G729 A 8,000 1
            } else if ( (new AudioFormat(AudioFormat.GSM_RTP))
                        .isSameEncoding(af) ) {
                return "3"; //RFC1890 RTP/AVP 3=GSM A 8000 1
            } else if ( (new AudioFormat(AudioFormat.MPEG_RTP))
                        .isSameEncoding(af) ) {
                return "14"; //RFC1890 RTP/AVP 14=MPA A 90000
            } else {
                return "-1"; //Formato desconocido
            }
        }
    }

    /**
     * Devuelve el tipo de carga RTP/AVP, el formato de audio segun RFC1890
     * @return String Payload Type RTP/AVP segun RFC1890
     */
    public String videoPT() {
        VideoFormat vf=null;

        if ( (video==null) || (!hayFormatoVideo) ) return null;
        else {
            for (int i=0; i<video.getTrackControls().length; i++) {
                if (video.getTrackControls()[i].isEnabled()) {
                    vf=(VideoFormat)(video.getTrackControls()[i].getFormat());
                }
            }
            if (vf==null) vf=new VideoFormat(formatoVideo.getSelectedItem());

            if ( (new VideoFormat(VideoFormat.JPEG_RTP)).isSameEncoding(vf) ) {
                return "26"; //RFC1890 RTP/AVP 26=JPEG V 90000
            } else if ( (new VideoFormat(VideoFormat.MPEG_RTP))
                        .isSameEncoding(vf) ) {
                return "32"; //RFC1890 RTP/AVP 32=MPV V 90000
            } else if ( (new VideoFormat(VideoFormat.H261_RTP))
                        .isSameEncoding(vf) ) {
                return "31"; //RFC1890 RTP/AVP 31=H261 V 90000
            } else if ( (new VideoFormat(VideoFormat.H263_RTP))
                        .isSameEncoding(vf) ) {
                return "34"; //RFC3551 RTP/AVP 34=H263 V 90,000
            } else if ( (new VideoFormat(VideoFormat.H263_1998_RTP))
                        .isSameEncoding(vf) ) {
                return "34"; //RFC3551 RTP/AVP 34=H263 V 90,000
            } else {
                return "-1"; //Formato desconocido
            }
        }
    }

    /**
     * Lanza una ventana con la informacion referente a la Transmision
     */
    public void infoTX() {
        String info=idioma.texto(2211)+": "+RETURN+RETURN;
        AudioFormat af=null;
        VideoFormat vf=null;

        //Informacion de Audio
        if ( (audio!=null) && (hayFormatoAudio) ) {
            for (int i=0; i<audio.getTrackControls().length; i++) {
                if (audio.getTrackControls()[i].isEnabled()) {
                    af=(AudioFormat)(audio.getTrackControls()[i].getFormat());
                }
            }
            if (af==null) af=new AudioFormat(formatoAudio.getSelectedItem());
        } else af=null;

        info=info+idioma.texto(2914)+": ";
        if (af==null) info=info+idioma.texto(2212);
        else info=info+af.getEncoding();
        info=info+RETURN;
        info=info+idioma.texto(2213);
        if (af==null) info=info+"---"+RETURN;
        else info=info+af.getSampleRate()+" Hz"+RETURN;
        info=info+idioma.texto(2214);
        if (af==null) info=info+"---"+RETURN;
        else info=info+af.getSampleSizeInBits()+" bits"+RETURN;
        info=info+idioma.texto(2215);
        if (af==null) info=info+"---"+RETURN+RETURN;
        else info=info+(af.getSampleRate()*af.getSampleSizeInBits())
                  +" bps"+RETURN+RETURN;

        //Informacion de video
        if ( (video!=null) && (hayFormatoVideo) ) {
            for (int i=0; i<video.getTrackControls().length; i++) {
                if (video.getTrackControls()[i].isEnabled()) {
                    vf=(VideoFormat)(video.getTrackControls()[i].getFormat());
                }
            }
            if (vf==null) vf=new VideoFormat(formatoVideo.getSelectedItem());
        } else vf=null;

        info=info+idioma.texto(2916)+": ";
        if (vf==null) info=info+idioma.texto(2221);
        else info=info+vf.getEncoding();
        info=info+RETURN;
        info=info+idioma.texto(2222);
        if (vf==null) info=info+"---"+RETURN;
        else info=info+vf.getFrameRate()+" Hz"+RETURN;

        info=info+idioma.texto(2223);
        if (vf==null) info=info+"---"+RETURN;
        else info=info+vf.getSize().height+idioma.texto(2224)+RETURN;
        info=info+idioma.texto(2225);
        if (vf==null) info=info+"---"+RETURN;
        else info=info+vf.getSize().width+idioma.texto(2224)+RETURN;
        info=info+idioma.texto(2226);
        if (vf==null) info=info+"---"+RETURN;
        else info=info+(vf.getSize().height*vf.getSize().width)
                  +idioma.texto(2224)+RETURN;
        info=info+idioma.texto(2227);
        if (vf==null) info=info+"---"+RETURN+RETURN;
        else info=info+(vf.getSize().height*vf.getSize().width)
                  +idioma.texto(2228)+RETURN+RETURN;

        //Pantalla de informacion
        JOptionPane.showMessageDialog(getContentPane(),info,idioma.texto(2211),
                                      JOptionPane.PLAIN_MESSAGE);
    }

    /**
     * Destructor de Preferencias.
     * Se libera la memoria ocupada y se cierra la Ventana.
     */
    public void salir() {
        setVisible(false);
	matarProcessor(audio);
        matarProcessor(video);
        matarProcessor(audioTemp);
        matarProcessor(videoTemp);
        dispose();
    }

}
