/**
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 * 
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */
/**
 * class Presentity
 * 
 * Presentity represents a presence uri
 * 
 * 
 */
package org.mitre.jsip.presence;

import org.mitre.jsip.*;
import java.util.*;

public class Subscription extends Object
{
    // class variables
    private long expires;
    private Date createTime;
    private Presentity presentity;

    private static final long DEFAULT_EXPIRES = 3600;

    public Subscription( Presentity presentity, long expires )
    {
	this.presentity = presentity;
	this.expires = expires;
	createTime = new Date();
    }

    public Subscription( Presentity presentity ) {
	// subscribe with default expiration 
	this( presentity, DEFAULT_EXPIRES );
    }

    public void renew( long expires )
    {
	this.expires = expires;
	createTime = new Date();
    }

    /**
     * Renew subscription with default expiration.
     */
    public void renew( ) 
    {
	renew( DEFAULT_EXPIRES );
    }

    public boolean hasExpired() {
	Date currentTime = new Date();

	return ( ( currentTime.getTime() - createTime.getTime() ) > expires );
    }

    public String toString() {
	// override Object's toString()
	return "Subscription";
    }

    public boolean equals( Object obj ) {
	// override Object's equals()
	return false;
    }
}
