/*
 * This file is part of the Java SIP library, an open source SIP library. Please see
 * http://collaboration.mitre.org/sip for more information
 *
 * Copyright (c) 2001 The MITRE Corporation
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */
/**
 * Class to initiate a thread to handle messages
 *
 * @author S.R.Jones
 * @version 1
 */

package org.mitre.jsip;

import org.mitre.jsip.event.*;

public class MessageThread
    extends Thread {
    private SipClient client;
    private static final int POLLTIMER=3000; // this is how often we'll check for messages (in msec)

    public MessageThread() {
        client=SipClient.getSipClient();
    }

    public void run() {
        // die if SipClient is no good
        if (client==null) {
            return;
        }

        // run this in a loop
        for (; ; ) {
            while (!client.haveMessages()) {
                try {
                    Thread.currentThread().sleep(POLLTIMER);
                }
                catch (InterruptedException ie) {
                    break;
                }
            }

            // there's a message available. send it to the handler
            client.incomingMessage();
        }
    }
}
