/*
 * This file was derived from libdissipate, an open source SIP library. The original file
 * was modified on 1/23/2001.  Please see
 * http://www.div8.net/dissipate for more information.
 *
 * Copyright (c) 2000 Billy Biggs <bbiggs@div8.net>
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */
/**
 * Class representing a SIP registration server and a users status with it.
 *
 * @author S.R.Jones
 * @version 1
 */

/*
 * This code has been generated using C2J++
 * C2J++ is based on Chris Laffra's C2J (laffra@watson.ibm.com)
 * Read general disclaimer distributed with C2J++ before using this code
 * For information about C2J++, send mail to Ilya_Tilevich@ibi.com
 */
package org.mitre.jsip;

import org.mitre.jsip.event.*;
import org.mitre.jsip.util.*;
import java.util.*;
import java.text.*;

public class SipRegister
    extends Object
    implements PollTask {
    /**
     * SipRegister
     * @param user
     * @param serveruri
     */
    public SipRegister(SipUser user, SipUri serveruri) {
        rstate=NotConnected;
        regop=NoRegOpPending;
        authtype=BasicAuthenticationRequired;
        tryCounter=0;

        this.user=user;
        regcall=new SipCall(user, null, SipCall.RegisterCall);
        regcall.setSubject("Register Call");

        final SipRegister myRegister=this;
        CallListener cl=new CallListener() {
            public void responseReceived(CallEvent ce) {
                SipResponseMessage respMessage= (SipResponseMessage)
                                               ce.getMessage();
                String respTo=respMessage.getHeaderData(SipHeader.
                    CSeq);

                // see if this is a response to a REGISTER request
                if (respTo.indexOf(Sip.getMethodString(Sip.REGISTER))<
                    0) {
                    return;
                }

                int retCode=respMessage.getStatus().getCode();
                if (retCode>=400) {
                    // some kind of error message occured
                    System.err.println("Re-register failed");
                    // TODO: a popup or something
                }
                else if (retCode==200) {
                    // Get the Expires header and set up next run
                    String expires=respMessage.getHeaderData(
                        SipHeader.Expires);
                    if (expires!=null) {
                        try {
                            SimpleDateFormat sdf=new SimpleDateFormat(
                                "EEE, dd MMM yyyy HH:mm:ss zzzz");

                            Date expirationDate=sdf.parse(expires);
                            if (expirationDate!=null) {
                                Date dateNow=new Date();
                                // Let's reregister when we're nearing expiration
                                // for now, let's say 90% expired
                                long reregTime= ( (long) ( (
                                    expirationDate.getTime()-
                                    dateNow.getTime())*
                                    EXPIRATION_LENGTH)+
                                                 dateNow.getTime());
                                regPoller.schedule(new Date(reregTime));
                            }
                        }
                        catch (ParseException pe) {
                            System.err.println("Error: "+pe);
                        }
                    }
                }

                // update status
                try {
                    localStatusUpdated(respMessage);
                }
                catch (RegisterException re) {
                    // hmm, shouldn't happen
                    System.err.println("Something weird happened: "+
                                       re);
                }

            }

            public void requestReceived(CallEvent ce) {}
        };
        // add this call listener to the client
        SipClient.getSipClient().addCallListener(cl);

        setServerUri(serveruri);

        // now set up polling to allow us to re-register
        PollAgent pa=new PollAgent();
        pa.addTask(this);

        regPoller=new Poller(pa);
    }

    /**
     * getRegisterState
     * @return RegisterState
     */
    public int getRegisterState() {
        return rstate;
    }

    /**
     * getAuthenticationType
     * @return AuthType
     */
    public int getAuthenticationType() {
        return authtype;
    }

    /**
         * Sets the authentication required as detailed in a response message
     */
    public void setAuthentication(SipResponseMessage respMessage) throws
        RegisterException {
        regop=NoRegOpPending;
        localStatusUpdated(respMessage);
    }

    /**
     * getServerUri
     * @return SipUri
     */
    public SipUri getServerUri() {
        return regserver;
    }

    /**
     * setServerUri
     * @param newregserver
     */
    public void setServerUri(SipUri newregserver) {
        regserver=newregserver;
    }

    /**
         * Request registration using already provided username and password
     */
    public void requestRegister(boolean clear) throws
        RegisterException {
        requestRegister(null, clear);
    }

    /**
     * Requests registration from the proxy.
     */
    public void requestRegister(String password) throws
        RegisterException {
        requestRegister(password, false);
    }

    public void requestRegister(String newPassword, boolean clear) throws
        RegisterException {
        // we try only so many times. After that, quit
        if (tryCounter>=REGISTER_ATTEMPTS) {
            regop=NoRegOpPending;
            tryCounter=0;
            throw new RegisterException("Failed to register after "+
                                        REGISTER_ATTEMPTS+" attempts");
        }

        tryCounter++;

        // keep these if we need them again
        String username=user.name();

        if (newPassword!=null) {
            this.password=newPassword;
        }

        // sortofa hack
        if (regop==NoRegOpPending) {
            regop=RegRequest;

        }
        if (rstate!=TryingServer) {

            if (rstate==AuthenticationRequired) {

                if (authtype==DigestAuthenticationRequired) {
                    // make sure we've received a response
                    if (authstr==null) {
                        return;
                    }

                    // the authstr should be "Digest [params], so cut off Digest
                    authstr=authstr.trim();
                    int spIdx=authstr.indexOf(" ");
                    if (spIdx<0) {
                        return;
                    }

                    DigestAuthentication da=new DigestAuthentication(
                        authstr.substring(spIdx+1));
                    try {
                        authresponse=da.createAuthorizationHeader(
                            username,
                            password,
                            "REGISTER",
                            regserver.uri());
                    }
                    catch (DigestAuthenticationException dae) {
                        System.err.println(
                            "Couldn't calculate Digest Auth: "+
                            dae.toString());
                        return;
                    }
                    //		    authresponse = Sip.getDigestResponse( username, password, "REGISTER",  regserver.uri(), authstr );
                }
                else if (authtype==BasicAuthenticationRequired) {
                    authresponse=Sip.getBasicResponse(username,
                        password);
                }
                else if (authtype==ProxyDigestAuthenticationRequired) {
                    proxyauthresponse=Sip.getDigestResponse(username,
                        password, "REGISTER", regserver.uri(),
                        proxyauthstr);
                }
                else if (authtype==ProxyBasicAuthenticationRequired) {
                    proxyauthresponse=Sip.getBasicResponse(username,
                        password);
                }
            }

            rstate=TryingServer;
            curtrans=regcall.newRegister(regserver, clear,
                                         authresponse,
                                         proxyauthresponse, null, null);

        }
        //   localStatusUpdated();
    }

    /**
     * Disable a registration to a Registrar server.
         * Does this by sending a REGISTER with "Expires: 0" to the server
     */
    public void requestClearRegistration() {
        if (regop==NoRegOpPending) {
            rstate=Disconnecting;
            regop=RegClear;

            curtrans=regcall.newRegister(regserver, true);
            //	    connect( curtrans, SIGNAL( statusUpdated() ), this, SLOT( localStatusUpdated() ) );
        }

        // let the poller know to stop
        regPoller.cancel();

        //statusUpdated();
        localStatusUpdated();
    }

    /**
     * getRegisterCall
     * @return SipCall *
     */
    public SipCall getRegisterCall() {
        return regcall;
    }

    /**
     * getAuthRealm
     * @return String
     */
    public String getAuthRealm() {
        String realm;

        if (authstr.indexOf("realm=\"")>=0) {
            realm=authstr.substring(authstr.indexOf("realm=\"")+7);
            realm=realm.substring(0, realm.indexOf('\"'));
            return realm;
        }

        return null;
    }

    /**
     * getProxyAuthRealm
     * @return String
     */
    public String getProxyAuthRealm() {
        String realm;

        if (proxyauthstr.indexOf("realm=\"")>=0) {
            realm=proxyauthstr.substring(authstr.indexOf("realm=\"")+
                                         7);
            realm=realm.substring(0, realm.indexOf('\"'));
            return realm;
        }

        return null;
    }

    /**
     * PollTask interface implementation
     */
    public void activate() {
        regop=ReRegRequest;
        try {
            requestRegister(false);
        }
        catch (RegisterException re) {
            System.err.println("Error registering: "+re);
        }
    }

    /**
     * statusUpdated
     */
    //private void statusUpdated() {
    //	SipCall.statusUpdated();
    //}

    /**
     * Update the register status depending on the response message received
     */
    private void localStatusUpdated(SipResponseMessage resp) throws
        RegisterException {

        String authstrtemp;
        String username=user.name();

        // switch on the response code
        switch (resp.getStatus().getCode()) {
            case (200):
                switch (regop) {
                    case RegRequest:
                        rstate=Connected;
                        break;
                    case RegClear:
                        rstate=NotConnected;
                        break;
                    default:
                        break;
                }
                tryCounter=0;
                regop=NoRegOpPending;
                break;
            case (401):
                if (rstate==AuthenticationRequired) {
                    // ok. try this:
                    // if this is a RegRequest (first try), throw the exception
                    // if this is a ReRegRequest, we've probably already registered
                    // successfully once before, so try again
                    if (regop==RegRequest) {
                        throw new RegisterException(
                            "Unauthorized username/password");
                    }
                    else if (regop==ReRegRequest) {
                        setAuthentication(resp);
                        activate();
                    }
                }

                rstate=AuthenticationRequired;

                authstr=resp.getHeaderData(SipHeader.WWW_Authenticate);

                // hmm, vocal doesn't send this if it doesn't want you in!
                if (authstr==null) {
                    return;
                }

                authstrtemp=authstr.toLowerCase();

                if (authstrtemp.indexOf("digest")>=0) {
                    authtype=DigestAuthenticationRequired;
                }
                else {
                    authtype=BasicAuthenticationRequired;
                }
                break;
            case (407):
                if (rstate==AuthenticationRequired) {
                    // we tried and failed again
                    throw new RegisterException(
                        "Unauthorized Proxy username/password");
                }

                rstate=AuthenticationRequired;

                proxyauthstr=curtrans.getFinalProxyAuthString();

                authstrtemp=proxyauthstr.toLowerCase();

                if (authstrtemp.indexOf("digest")>=0) {
                    authtype=ProxyDigestAuthenticationRequired;
                }
                else {
                    authtype=ProxyBasicAuthenticationRequired;
                }
                break;
            default:
                switch (regop) {
                    case RegRequest:
                        rstate=NotConnected;
                        break;
                    case RegClear:
                        rstate=Connected;
                        break;
                    default:
                        break;
                }
        }
    }

    /**
     * localStatusUpdated
     */
    private void localStatusUpdated() {
        System.out.println("REGCALLLISTENER in lSU: status is "+
                           curtrans.getStatus().getCode());
        String authstrtemp;

        SipStatus status=curtrans.getStatus();
        if ( (status!=null)&& (status.getCode()>=200)) { // final response

            if (curtrans.getStatus().getCode()==200) {

                switch (regop) {
                    case RegRequest:
                        rstate=Connected;
                        break;
                    case RegClear:
                        rstate=NotConnected;
                        break;
                    default:
                        break;
                }

            }
            else if (curtrans.getStatus().getCode()==401) {

                rstate=AuthenticationRequired;

                authstr=curtrans.getFinalWWWAuthString();

                authstrtemp=authstr.toLowerCase();

                if (authstrtemp.indexOf("digest")>=0) {
                    authtype=DigestAuthenticationRequired;
                }
                else {
                    authtype=BasicAuthenticationRequired;
                }

            }
            else if (curtrans.getStatus().getCode()==407) {

                rstate=AuthenticationRequired;

                proxyauthstr=curtrans.getFinalProxyAuthString();

                authstrtemp=proxyauthstr.toLowerCase();

                if (authstrtemp.indexOf("digest")>=0) {
                    authtype=ProxyDigestAuthenticationRequired;
                }
                else {
                    authtype=ProxyBasicAuthenticationRequired;
                }

            }
            else {

                switch (regop) {
                    case RegRequest:
                        rstate=NotConnected;
                        break;
                    case RegClear:
                        rstate=Connected;
                        break;
                    default:
                        break;
                }

            }
        }

        regop=NoRegOpPending;
        //	statusUpdated();
    }

    // class variables
    static final int NotConnected=0;
    static final int TryingServer=1;
    static final int AuthenticationRequired=2;
    static final int Connected=3;
    static final int Disconnecting=4;
    static final int BasicAuthenticationRequired=0;
    static final int DigestAuthenticationRequired=1;
    static final int ProxyBasicAuthenticationRequired=2;
    static final int ProxyDigestAuthenticationRequired=3;

    static final int NoRegOpPending=0;
    static final int RegRequest=1;
    static final int RegClear=2;
    static final int ReRegRequest=3;

    private static final int REGISTER_ATTEMPTS=3;
    private static final double EXPIRATION_LENGTH=.9;

    private SipUser user;
    private SipCall regcall;
    private SipTransaction curtrans;
    private int regop;

    private String password;
    private Timer reRegisterTimer;
    private Poller regPoller;

    static int tryCounter;

    private CallListener regCallListener;
    int rstate;
    int authtype;
    SipUri regserver;
    String authstr;
    String proxyauthstr;
    String authresponse;
    String proxyauthresponse;

}
