/*
 * This file was derived from libdissipate, an open source SIP library. The original file
 * was modified on 1/23/2001.  Please see
 * http://www.div8.net/dissipate for more information.
 *
 * Copyright (c) 2000 Billy Biggs <bbiggs@div8.net>
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */
/**
 * class SipUri
 *
 * Class to parse and reference a complete SIP URI
 *
 * This code has been generated using C2J++
 * C2J++ is based on Chris Laffra's C2J (laffra@watson.ibm.com)
 * Read general disclaimer distributed with C2J++ before using this code
 * For information about C2J++, send mail to Ilya_Tilevich@ibi.com
 */
package org.mitre.jsip;

import java.util.Random;
import java.io.Serializable;

public class SipUri
    extends GeneralUri
    implements Serializable {

    /**
     * SipUri
     */
    public SipUri() {
        clear();
    }

    /**
     * SipUri
     * @param parseinput
     */
    public SipUri(String parseinput) {
        clear();
        if (parseinput!=null) {
            parseUri(parseinput);
        }
    }

    /**
     * isValid
     * @return bool
     */
    public boolean isValid() {
        return isvalid;
    }

    /**
     * getSipPort
     * @return int
     */
    public int getSipPort() {
        return 5060;
    }

    /**
     * getUserParamString
     * @param u
     * @return String
     */
    public static String getUserParamString(int u) {
        switch (u) {
            case Phone:
                return "phone";
            case IP:
                return "ip";
            case NoUserParam:
                return "BAD";
        }
        ;
        return null;
    }

    /**
     * getTransportParamString
     * @param t
     * @return String
     */
    public static String getTransportParamString(int t) {
        switch (t) {
            case UDP:
                return "udp";
            case TCP:
                return "tcp";
            case NoTransportParam:
                return "BAD"; // Should return udp
        }
        ;
        return null;
    }

    /**
     * getProtocolName
     * @return String
     */
    public String getProtocolName() {
        return protocol;
    }

    /**
     * Set the protocol for this URI
     *
     * @param name The protocol name
     */
    public void setProtocolName(String name) {
        protocol=name;
    }

    /**
     * Print a uri. By default, will not display method params
     */
    public void setShowMethods(boolean showMethods) {
        this.showMethods=showMethods;
    }

    /**
     * Print the user's uri.
     *
     * @param showMethods Determines whether to display the methods supported
     * @return String
     */
    public String uri() {
        StringBuffer uritext;

        uritext=new StringBuffer(protocol+":");

        if (hasUserInfo()) {
            uritext.append(username);
            if (hasPassword()) {
                uritext.append(":"+password);
            }
            uritext.append("@");
        }

        uritext.append(hostname);
        if (hasport||port!=getSipPort()) {
            uritext.append(":"+String.valueOf(port));
        }

        if (hasTransportParam()) {
            uritext.append(";transport="
                           +getTransportParamString(transparam));
        }

        if (hasUserParam()) {
            uritext.append(";user="+getUserParamString(userparam));
        }

        if ( (showMethods)&& (hasMethodsParam())) {
            // determine which methods we have available
            StringBuffer methodBuffer=new StringBuffer(";methods=\"");
            if ( (methods&METHOD_MESSAGE)==METHOD_MESSAGE) {
                methodBuffer.append("MESSAGE,");
            }
            if ( (methods&METHOD_SUBSCRIBE)==METHOD_SUBSCRIBE) {
                methodBuffer.append("SUBSCRIBE,");
            }
            if ( (methods&METHOD_NOTIFY)==METHOD_NOTIFY) {
                methodBuffer.append("NOTIFY,");
            }
            if ( (methods&METHOD_INVITE)==METHOD_INVITE) {
                methodBuffer.append("INVITE,");

                // replace the last comma with an endquote
            }
            int lastpos=methodBuffer.length()-1;
            methodBuffer.setCharAt(lastpos, '\"');
            uritext.append(methodBuffer.toString());
        }

        if (hasTtlParam()) {
            uritext.append(";ttl="+String.valueOf( (int) ttl));
        }

        if (hasMaddrParam()) {
            uritext.append(";maddr="+maddrhostname);
        }

        return uritext.toString();
    }

    /**
     * nameAddr
     * @return String
     */
    public String nameAddr() {
        StringBuffer nameaddr=new StringBuffer();
        boolean hasFullName=false;

        if ( (fullname!=null)&& (fullname.length()>0)) {
            hasFullName=true;
            // if we see a double quote, assume string is quoted and we don't need to do it
            if (fullname.indexOf("\"")>=0) {
                nameaddr.append(fullname+" <");
            }
            else {
                nameaddr.append("\""+fullname+"\" <");
            }
        }

        nameaddr.append(uri());
        if (hasFullName) {
            nameaddr.append(">");

        }
        if (hasTag()) {
            nameaddr.append(";tag="+getTag());
        }

        return nameaddr.toString();
    }

    /**
     * reqUri
     * @return String
     */
    public String reqUri() {
        String uritext;

        uritext=protocol+":";

        if (hasUserInfo()) {
            uritext+=username+"@";
            if (hasPassword()) {
                uritext+=password;
            }
        }

        uritext+=hostname;
        if (hasport||port!=getSipPort()) {
            uritext+=":"+String.valueOf(port);
        }

        return uritext;
    }

    /**
     * user
     * @return String
     */
    public String user() {
        StringBuffer usertext=new StringBuffer();

        if (hasUserInfo()) {
            usertext.append(username);
            if (hasPassword()) {
                usertext.append(":"+password);
            }
        }

        usertext.append("@"+hostname);

        return usertext.toString();
    }

    /**
     * getFullname
     * @return String
     */
    public String getFullname() {
        return fullname;
    }

    /**
     * getUsername
     * @return String
     */
    public String getUsername() {
        return username;
    }

    /**
     * getPassword
     * @return String
     */
    public String getPassword() {
        return password;
    }

    /**
     * getHostname
     * @return String
     */
    public String getHostname() {
        return hostname;
    }

    /**
     * getTag
     * @return String
     */
    public String getTag() {
        return tag;
    }

    /**
     * generateTag
     */
    public void generateTag() {
        int uniqid;
        Random rand=new Random();

        uniqid=rand.nextInt(6000);

        tag=String.valueOf(uniqid);

        hastag=true;
    }

    /**
     * setTag
     * @param newtag
     */
    public void setTag(String newtag) {
        tag=newtag;

        if (tag==null) {
            hastag=false;
        }
        else {
            hastag=true;
        }
    }

    /**
     * setFullname
     * @param newfname
     */
    public void setFullname(String newfname) {
        if (newfname==null) {
            return;
        }

        fullname=newfname.trim();

        // take out any '\'        *** LOOK AT ****

        if ( (fullname.indexOf('\\'))>=0) {
            StringBuffer sb=new StringBuffer(fullname);
            int index;
            while ( (index=fullname.indexOf('\\'))>=0) {
                sb.replace(index, index, "");
            }
            fullname=sb.toString();
        }
    }

    /**
     * setUsername
     * @param u
     */
    public void setUsername(String u) {
        if (u==null) {
            hasuserinfo=false;
        }
        else {
            hasuserinfo=true;
            username=u;
        }
    }

    /**
     * setHostname
     * @param hname
     */
    public void setHostname(String hname) {
        hostname=hname.toLowerCase().trim();
    }

    /**
     * setPassword
     * @param p
     */
    public void setPassword(String p) {
        if (p==null) {
            haspassword=false;
        }
        else {
            haspassword=true;
            password=p;
        }
    }

    /**
     * hasUserInfo
     * @return bool
     */
    public boolean hasUserInfo() {
        return hasuserinfo;
    }

    /**
     * hasPassword
     * @return bool
     */
    public boolean hasPassword() {
        return haspassword;
    }

    /**
     * setPortNumber
     * @param p
     */
    public void setPortNumber(int p) {
        port=p;
    }

    /**
     * getPortNumber
     * @return int
     */
    public int getPortNumber() {
        return (hasport)?port:getSipPort();
    }

    /**
     * hasTransportParam
     * @return bool
     */
    public boolean hasTransportParam() {
        return hastransparam;
    }

    /**
     * setTransportParam
     * @param t
     */
    public void setTransportParam(int t) {
        if (t==NoTransportParam) {
            hastransparam=false;
        }
        else {
            hastransparam=true;
            transparam=t;
        }
    }

    /**
     * hasUserParam
     * @return bool
     */
    public boolean hasUserParam() {
        return hasuserparam;
    }

    /**
     * setUserParam
     * @param u
     */
    public void setUserParam(int u) {
        if (u==NoUserParam) {
            hasuserparam=false;
        }
        else {
            hasuserparam=true;
            userparam=u;
        }
    }

    public int getUserParam() {
        return userparam;
    }

    /**
     * hasMethodParam
     * @return bool
     */
    public boolean hasMethodsParam() {
        return methods>0;
    }

    /**
     * setMethodParam
     * @param m
     */
    public void setMethodsParam(int m) {
        if (m==Sip.BadMethod) {
            return;
        }

        methods|=m;
    }

    /**
     * hasTtlParam
     * @return bool
     */
    public boolean hasTtlParam() {
        return hasttlparam;
    }

    /**
     * setTtl
     * @param t
     */
    public void setTtl(int t) {
        ttl=t;
    }

    public int getTtlParam() {
        return ttl;
    }

    /**
     * hasMaddrParam
     * @return bool
     */
    public boolean hasMaddrParam() {
        return hasmaddrparam;
    }

    /**
     * getMaddrParam
     * @return String
     */
    public String getMaddrParam() {
        return maddrhostname;
    }

    /**
     * setMaddrParam
     * @param newmaddr
     */
    public void setMaddrParam(String newmaddr) {
        if (newmaddr!=null) {
            maddrhostname=newmaddr;
            hasmaddrparam=true;
        }
        else {
            hasmaddrparam=false;
            maddrhostname=null;
        }
    }

    /**
     * hasTag
     * @return bool
     */
    public boolean hasTag() {
        return hastag;
    }

    /**
     * equals
     */
    public boolean equals(Object obj) {
        SipUri suri=null;
        try {
            suri= (SipUri) obj;
        }
        catch (ClassCastException cce) {
            return false;
        }

        if (!isValid()||!suri.isValid()) {
            return false;
        }

        return ( (username.equals(suri.getUsername()))&&
                (hostname.equals(suri.getHostname()))&&
                (getPortNumber()==suri.getPortNumber()));

    }

    // class variables
    private String protocol="sip";
    private String fullname;
    private boolean hasuserinfo;
    private String username;
    private boolean haspassword;
    private String password;
    private int port;
    private boolean isvalid;
    private boolean hastransparam;
    private int transparam;
    private boolean hasuserparam;
    private int userparam;
    private int methods;
    private boolean hasttlparam;
    private int ttl;
    private boolean hasmaddrparam;
    private String maddrhostname;
    private boolean hastag;
    private boolean hasport;
    private String tag;
    private String hostname;

    private boolean showMethods;
    /**
     * clear
     */
    private void clear() {
        isvalid=true;
        hasuserinfo=false;
        haspassword=false;
        port=getSipPort();
        hastransparam=false;
        hasuserparam=false;
        hasttlparam=false;
        ttl=0;
        hasmaddrparam=false;
        hastag=false;
        hasport=false;

        // items in equals() that could cause problems
        username="";
        hostname="";
        password="";
    }

    /**
     * parseUri
     * @param parseinput
     */
    protected void parseUri(String parseinput) {
        String inputline;
        String userinfo;
        String hostport;
        String parameters;

        isvalid=false;

        inputline=parseinput.trim();

        // Squeeze out blah<uri>blah
        if (inputline.indexOf('<')>=0) {

            // Validity check (possibly bogus)
            if (inputline.indexOf('>')==-1) {
                return;
            }

            // Parse out the squeezed uri
            String beforeangle=inputline.substring(0,
                inputline.indexOf('>'));
            parseUri(beforeangle.substring(beforeangle.indexOf('<')+1));

            // Parse the fullname, if any
            setFullname(inputline.substring(0, inputline.indexOf('<')));

            // Parse the parameters, if any
            parseParameters(inputline.substring(inputline.indexOf('>')+
                                                1));

            return;
        }

        // Take out the protocol prefix if it's there
        if (inputline.toLowerCase().startsWith("sip:")||
            inputline.toLowerCase().startsWith("im:")) {
            int colonPos=inputline.indexOf(':');
            protocol=inputline.substring(0, colonPos);
            inputline=inputline.substring(colonPos+1);
        }

        // If we have user info
        if (inputline.indexOf('@')>=0) {
            userinfo=inputline.substring(0, inputline.indexOf('@'));

            if (userinfo.indexOf(':')>=0) {
                setUsername(userinfo.substring(0,
                                               userinfo.indexOf(':')));
                setPassword(userinfo.substring(userinfo.indexOf(':')+
                                               1));
            }
            else {
                setUsername(userinfo);
            }
            inputline=inputline.substring(inputline.indexOf('@')+1);
        }

        // If we have any parameters
        if (inputline.indexOf(';')>=0) {
            hostport=inputline.substring(0, inputline.indexOf(';')).
                     trim();
            inputline=inputline.substring(inputline.indexOf(';'));
        }
        else if (inputline.indexOf('?')>=0) {
            hostport=inputline.substring(0, inputline.indexOf('?')).
                     trim();
            inputline=inputline.substring(inputline.indexOf('?')+1);
        }
        else {
            hostport=inputline;
            inputline=""; // *** LOOK AT ***
        }

        if (hostport.indexOf(':')>=0) {
            setHostname(hostport.substring(0, hostport.indexOf(':')));
            try {
                String sPortNum=hostport.substring(hostport.indexOf(
                    ':')+1);
                int portNum=Integer.parseInt(sPortNum);
                setPortNumber(portNum);
                hasport=true;
            }
            catch (NumberFormatException nfe) {
                System.err.println(
                    "Couldn't parse port number, setting to standard value");
                setPortNumber(getSipPort());
            }
        }
        else {
            setHostname(hostport);
            setPortNumber(getSipPort());
        }

        if (inputline.indexOf('?')>=0) {
            parameters=inputline.substring(0, inputline.indexOf('?'));
            inputline=inputline.substring(inputline.indexOf('?')+1);
        }
        else {
            parameters=inputline;
            inputline=null;
        }

        // Parse the parameters at the end
        parseParameters(parameters);

        // If we make it this far
        isvalid=true;
    }

    /**
     * parseParameters
     * @param parseinput
     */
    private void parseParameters(String parseinput) {
        String curparm;
        String parmname;
        String parmvalue;
        String parameters;

        // NOTE: Calling this function will never change validity of the URI

        parameters=parseinput.trim();

        while (parameters.startsWith(";")) {
            curparm=parameters.substring(1).trim();
            int location=parameters.indexOf(";", 1);
            if (location>0) {
                parameters=parameters.substring(1, location);
            }
            else {
                parameters=parameters.substring(1);

            }
            parmname=curparm.substring(0, curparm.indexOf('=')).
                     toLowerCase();
            parmvalue=curparm.substring(curparm.indexOf('=')+1).trim();

            // Check if the URI has a tag
            if (parmname.equals("tag")) {
                hastag=true;
                tag=parmvalue;
            }

            // maddr param
            if (parmname.equals("maddr")) {
                hasmaddrparam=true;
                maddrhostname=parmvalue.toLowerCase();
            }

            // ttl param
            else if (parmname.equals("ttl")) {
                hasttlparam=true;
                try {
                    ttl=Integer.parseInt(parmvalue);
                }
                catch (NumberFormatException nfe) {
                    System.err.println(
                        "Error formatting ttl, set to default value");
                    ttl=DEFAULT_TTL;
                }
            }

            // transport param
            else if (parmname.equals("transport")) {
                if (parmvalue.toLowerCase()=="udp") {
                    transparam=UDP;
                    hastransparam=true;
                }
                else if (parmvalue.toLowerCase()=="tcp") {
                    transparam=TCP;
                    hastransparam=true;
                }
            }
        }
    }

    /**
     * Test method for SipUri
     */
    public static void main(String[] args) {

        for (int i=0; i<args.length; i++) {
            SipUri sipUri=new SipUri(args[i]);

            // see what we got
            System.out.println("Fullname is "+sipUri.getFullname());
            System.out.println("Username is "+sipUri.getUsername());
            System.out.println("Hostname is "+sipUri.getHostname());
            System.out.println("Port is "+sipUri.getPortNumber());

            if (sipUri.hasTag()) {
                System.out.println("Tag is "+sipUri.getTag());
            }
            if (sipUri.hasMaddrParam()) {
                System.out.println("Multicast address is "+
                                   sipUri.getMaddrParam());
            }
            if (sipUri.hasTtlParam()) {
                System.out.println("TTL is "+sipUri.getTtlParam());
            }
            if (sipUri.hasUserParam()) {
                System.out.println("User parameter is "+
                                   SipUri.
                                   getUserParamString(sipUri.getUserParam()));
            }
            if (sipUri.hasTransportParam()) {
                System.out.println("Transport parameter is "+
                                   SipUri.
                                   getTransportParamString(sipUri.
                    getUserParam()));
            }
            if (sipUri.hasPassword()) {
                System.out.println("Password is "+sipUri.getPassword());

            }
            System.out.println("I would write this uri as:");
            System.out.println(sipUri.uri());

            System.out.println("And the name address as:");
            System.out.println(sipUri.nameAddr());
        }
        System.exit(0);

    }

    // class variables
    // UserParam
    static final int Phone=0;
    static final int IP=1;
    static final int NoUserParam=2;

    // TransportParam
    public static final int UDP=0;
    public static final int TCP=1;
    public static final int NoTransportParam=2;

    public static final int METHOD_MESSAGE=0x01;
    public static final int METHOD_SUBSCRIBE=0x02;
    public static final int METHOD_NOTIFY=0x04;
    public static final int METHOD_INVITE=0x08;

    static final int DEFAULT_TTL=15;
}
