/*
 * This file was derived from libdissipate, an open source SIP library. The original file
 * was modified on 1/23/2001.  Please see
 * http://www.div8.net/dissipate for more information.
 *
 * Copyright (c) 2000 Billy Biggs <bbiggs@div8.net>
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */
/**
 * class SipUser
 *
 * Class for describing a user of a SipClient.
 *
 * This code has been generated using C2J++
 * C2J++ is based on Chris Laffra's C2J (laffra@watson.ibm.com)
 * Read general disclaimer distributed with C2J++ before using this code
 * For information about C2J++, send mail to Ilya_Tilevich@ibi.com
 */
package org.mitre.jsip;

import java.util.*;

public class SipUser {

    /**
     * SipUser
     * @param parent
     * @param fullname
     * @param username
     * @param athostname
     */
    public SipUser(SipClient parent, String fullname, String username,
                   String athostname) {
        client=parent;

        myuri.setFullname(fullname);
        myuri.setUsername(username);
        myuri.setHostname(athostname);

        client.addUser(this);
    }

    /**
     * SipUser
     * @param parent
     * @param inituri
     */
    public SipUser(SipClient parent, SipUri inituri) {
        client=parent;

        myuri=inituri;

        client.addUser(this);
    }

    /**
     * addServer
     * @param servername
     * @return SipRegister*
     */
    public SipRegister addServer(String servername) {
        SipRegister server=new SipRegister(this,
                                           new SipUri(servername));
        servers.put(servername, server);
        return server;
    }

    /**
     * removeServer
     * @param server
     */
    public void removeServer(SipRegister server) {
        servers.remove(server);
    }

    /**
     * setUri
     * @param newuri
     */
    public void setUri(SipUri newuri) {
        myuri=newuri;
    }

    /**
     * getUri
     * @return SipUri &
     */
    public SipUri getUri() {
        return myuri;
    }

    /**
     * parent
     * @return SipClient *
     */
    public SipClient parent() {
        return client;
    }

    /**
     * getSipRegisterList
     * @return SipRegisterIterator
     */
    public Hashtable getSipRegisterList() {
        return servers;
    }

    /**
     * Return the name to be used for authentication
     */
    public String name() {
        return myuri.user();
    }

    // class variables
    private SipClient client;
    private SipUri myuri;
    private Hashtable servers=new Hashtable();

}
