/*
 * This file is part of the Java SIP library, an open source SIP library. Please see
 * http://collaboration.mitre.org/sip for more information
 *
 * Copyright (c) 2001 The MITRE Corporation
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */
/**
 * Defines the action when a response is received for a call
 */

package org.mitre.jsip.event;

import java.awt.*;
import org.mitre.jsip.*;

public class CallEvent
    extends AWTEvent {

    private SipMessage message;
    private Object source;

    public static final int NEXT_ID=AWTEvent.RESERVED_ID_MAX+1;

    public static final int CALLRESPONSE_ID=NEXT_ID;
    public static final int CALLREQUEST_ID=NEXT_ID+1;

    // do we want to keep these?
    public static final int REQUEST=0;
    public static final int RESPONSE=1;
    public static final int SHUTDOWN=2;

    public CallEvent(Object source, SipMessage message, int id) {
        super(message, id);
        this.source=source;
        this.message=message;
    }

    /**
     * Get the message that triggered this event
     *
     * @return the current SIP message
     */
    public SipMessage getMessage() {
        return message;
    }

    public String toString() {
        return getClass().getName()+"[message=\n"+message.message()+
            "]";
    }

    public Object getSource() {
        return source;
    }
}
