/**
 * class InviteMessage
 *
 * Representation of a SIP INVITE message.
 *
 */

package org.mitre.jsip.messages;

import org.mitre.jsip.*;

public class InviteMessage
    extends SipRequestMessage {

    /**
     * Constructor
     */
    public InviteMessage() {
        super();

        meth=Sip.INVITE;
    }

    /**
     * Create the first line of this message
     */
    protected String startLine() {
        return "INVITE "+requesturi.reqUri()+" "+getVersionString()+
            "\r\n";
    }

    /**
     * See if this is a valid message
     *
     */
    protected void validateMessage() throws BadHeaderException {
        // The message should have several manditory fields. Make
        // sure they're there
        final int[] manditoryHeaders= {SipHeader.Call_ID,
                                     SipHeader.CSeq,
                                     SipHeader.From, SipHeader.To,
                                     SipHeader.Content_Length};

        for (int i=0; i<manditoryHeaders.length; i++) {
            // check for their existance. If not there, throw the exception
            if (!hasHeader(manditoryHeaders[i])) {
                throw new BadHeaderException(
                    "Message is missing header: "+
                    SipHeader.getHeaderString(manditoryHeaders[i]));
            }
        }

        // make sure there's a "Via"
        if (vialist.size()==0) {
            throw new BadHeaderException(
                "Message is missing header: "+
                SipHeader.getHeaderString(SipHeader.Via));
        }
    }

    /**
     * Signal to the user that an invite has been received
     */
    public boolean handleNewRequest(SipClient client) {
        // TBD
        return false;
    }

    /**
     * Handle a request for a call that already exists.
     */
    public void handleCallRequest(SipClient client) {
    }
}
