/**
 * class MessageMessage
 *
 * Representation of a SIP MESSAGE message.
 *
 */

package org.mitre.jsip.messages;

import org.mitre.jsip.*;
import org.mitre.jsip.event.*;

public class MessageMessage
    extends SipRequestMessage {

    /**
     * Constructor
     */
    public MessageMessage() {
        super();

        meth=Sip.MESSAGE;
    }

    /**
     * Create the first line of this message
     */
    protected String startLine() {
        return "MESSAGE "+requesturi.reqUri()+" "+getVersionString()+
            "\r\n";
    }

    /**
     * See if this is a valid message
     *
     */
    protected void validateMessage() throws BadHeaderException {
        // The message should have several manditory fields. Make
        // sure they're there
        final int[] manditoryHeaders= {SipHeader.Call_ID,
                                     SipHeader.CSeq,
                                     SipHeader.From, SipHeader.To,
                                     SipHeader.Content_Length};

        for (int i=0; i<manditoryHeaders.length; i++) {
            // check for their existance. If not there, throw the exception
            if (!hasHeader(manditoryHeaders[i])) {
                throw new BadHeaderException(
                    "Message is missing header: "+
                    SipHeader.getHeaderString(manditoryHeaders[i]));
            }
        }

        // make sure there's a "Via"
        if (vialist.size()==0) {
            throw new BadHeaderException(
                "Message is missing header: "+
                SipHeader.getHeaderString(SipHeader.Via));
        }
    }

    /**
     * Signal new incoming message to the user
     */
    public boolean handleNewRequest(SipClient client) {
        //
//  	MessageListener messageListener = client.getMessageListener();

//  	if ( messageListener != null ) {
//  	    MessageEvent me = new MessageEvent( client, this,
//  						MessageEvent.MESSAGE_ID );
//  	    messageListener.messageReceived( me );
//  	}

        return false;
    }

    public void handleCallRequest(SipClient client) {
        // do a message event
//  	MessageListener messageListener = client.getMessageListener();

//  	if ( messageListener != null ) {
//  	    MessageEvent me = new MessageEvent( client, this,
//  						MessageEvent.MESSAGE_ID );
//  	    messageListener.messageReceived( me );
//  	}

    }
}
