/**
 * class NotifyMessage
 *
 * Representation of a SIP MESSAGE message.
 *
 */

package org.mitre.jsip.messages;

import org.mitre.jsip.*;
import org.mitre.jsip.event.*;

public class NotifyMessage
    extends SipRequestMessage {

    /**
     * Constructor
     */
    public NotifyMessage() {
        super();

        meth=Sip.NOTIFY;
    }

    /**
     * Create the first line of this message
     */
    protected String startLine() {
        return "NOTIFY "+requesturi.reqUri()+" "+getVersionString()+
            "\r\n";
    }

    /**
     * See if this is a valid message
     *
     */
    protected void validateMessage() throws BadHeaderException {
        // The message should have several manditory fields. Make
        // sure they're there
        final int[] manditoryHeaders= {SipHeader.Call_ID,
                                     SipHeader.CSeq,
                                     SipHeader.From, SipHeader.To,
                                     SipHeader.Content_Length};

        for (int i=0; i<manditoryHeaders.length; i++) {
            // check for their existance. If not there, throw the exception
            if (!hasHeader(manditoryHeaders[i])) {
                throw new BadHeaderException(
                    "Message is missing header: "+
                    SipHeader.getHeaderString(manditoryHeaders[i]));
            }
        }

        // make sure there's a "Via"
        if (vialist.size()==0) {
            throw new BadHeaderException(
                "Message is missing header: "+
                SipHeader.getHeaderString(SipHeader.Via));
        }
    }

    /**
     * Signal new incoming message to the user
     */
    public boolean handleNewRequest(SipClient client) {
        //
//  	PresenceListener presenceListener = client.getPresenceListener();

//  	if ( presenceListener != null ) {
//  	    PresenceEvent pe = new PresenceEvent( client, this,
//  						PresenceEvent.NOTIFY_ID );
//  	    presenceListener.notifyReceived( pe );
//  	}

        return false;
    }

    public void handleCallRequest(SipClient client) {
        // do a message event
//  	PresenceListener presenceListener = client.getPresenceListener();

//  	if ( presenceListener != null ) {
//  	    PresenceEvent pe = new PresenceEvent( client, this,
//  						PresenceEvent.NOTIFY_ID );
//  	    presenceListener.notifyReceived( pe );
//  	}

    }
}
