/**
 * class RegisterMessage
 *
 * Representation of a SIP REGISTER message.
 *
 */

package org.mitre.jsip.messages;

import org.mitre.jsip.*;

public class RegisterMessage
    extends SipRequestMessage {

    /**
     * Constructor
     */
    public RegisterMessage() {
        super();

        meth=Sip.REGISTER;
    }

    /**
     * Create the first line of this message
     */
    protected String startLine() {
        return "REGISTER "+requesturi.reqUri()+" "+getVersionString()+
            "\r\n";
    }

    /**
     * See if this is a valid message
     *
     */
    protected void validateMessage() throws BadHeaderException {
        // The message should have several manditory fields. Make
        // sure they're there
        final int[] manditoryHeaders= {SipHeader.Call_ID,
                                     SipHeader.CSeq,
                                     SipHeader.From, SipHeader.To,
                                     SipHeader.Content_Length};

        for (int i=0; i<manditoryHeaders.length; i++) {
            // check for their existance. If not there, throw the exception
            if (!hasHeader(manditoryHeaders[i])) {
                throw new BadHeaderException(
                    "Message is missing header: "+
                    SipHeader.getHeaderString(manditoryHeaders[i]));
            }
        }

        // make sure there's a "Via"
        if (vialist.size()==0) {
            throw new BadHeaderException(
                "Message is missing header: "+
                SipHeader.getHeaderString(SipHeader.Via));
        }
    }

    /**
     * Handle an incoming request. This UAS can't handle REGISTER
     */
    public boolean handleNewRequest(SipClient client) {
        System.err.println(
            "Received a REGISTER message! Returning 405");
        client.sendQuickResponse(this, new SipStatus(405));
        return true;
    }

    /**
     * Handle a request for a call that already exists. This UAS can handle REGISTER
     */
    public void handleCallRequest(SipClient client) {
        System.err.println(
            "Received a REGISTER erroneously. Returning 405");
        client.sendQuickResponse(this, new SipStatus(405));
    }
}
