/*
 * Copyright (c) 2001 The MITRE Corporation <srjones@mitre.org>
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

/**
 * Class to provide a means to lookup a Proxy Server
 *
 */

package org.mitre.jsip.proxy;

import org.mitre.jsip.*;
import java.util.*;

public class FileProxyLookup
    implements ProxyLookup {

    private Hashtable proxyCache=new Hashtable();
    private String localDomainName;

    /**
     * Initialize with local proxy host
     */
    public FileProxyLookup(String domainName, String hostName) {
        localDomainName=domainName;

        proxyCache.put(domainName, new SipUri("sip:"+hostName));
    }

    /**
     * A quick 'n dirty way to add more proxies
     */
    void addProxy(String domainName, String hostName) {
        proxyCache.put(domainName, new SipUri("sip:"+hostName));
    }

    /**
     * Return the local proxy server
     */
    public SipUri getProxyUri() throws SipProxyException {
        return (SipUri) proxyCache.get(localDomainName);
    }

    /**
     * Return a proxy server for a particular host ( or domain )
     */
    public SipUri getConnectionUri(String host) throws
        SipProxyException {
        if (proxyCache.containsKey(host)) {
            return (SipUri) proxyCache.get(host);
        }

        // assume this is a host
        return new SipUri("sip:"+host);
    }

    public static void main(String[] argc) {
        // create a new instance, then test returns
        String host1="mitre.org";

        FileProxyLookup fpl=new FileProxyLookup(host1,
                                                "stars.mitre.org");

        String host2="blah.mitre.com";

        try {
            System.out.println("For host "+host1+
                               " the returned proxy is "+
                               fpl.getConnectionUri(host1).
                               getHostname());

            System.out.println("For host "+host2+
                               " the returned proxy is "+
                               fpl.getConnectionUri(host2).
                               getHostname());
        }
        catch (SipProxyException spe) {
            System.err.println(spe.toString());
            System.exit(-1);
        }

        System.exit(0);
    }
}
