/*
 * Copyright (c) 2001 The MITRE Corporation <srjones@mitre.org>
 *
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to the
     * Free Software Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 */

/**
 * Initializes the proxy server lookup to use
 */

package org.mitre.jsip.proxy;

import org.mitre.jsip.*;
import java.lang.reflect.*;

public class ProxyServer {

    // do this for now. Will probably make these properties later on
    private static final String proxyLookupClass=
        "org.mitre.jsip.proxy.FileProxyLookup";

    private static final String localDomainName=
        "mitre.org";

    private static final String localProxyServer=
        "sturgis.mitre.org";

    private static ProxyLookup proxyLookup=null;

    protected ProxyServer() throws SipProxyException {

        try {
            Class[] classArgs= {String.class, String.class};
            Object[] initArgs= {localDomainName, localProxyServer};

            Class proxyClass=Class.forName(proxyLookupClass);
            Constructor construct=proxyClass.getConstructor(classArgs);

            // initialize
            proxyLookup= (ProxyLookup) construct.newInstance(initArgs);
        }
        catch (Exception e) {
            throw new SipProxyException(e.toString());
        }
    }

    public static ProxyLookup getProxyLookup() {

        if (proxyLookup==null) {
            try {
                ProxyServer ps=new ProxyServer();
            }
            catch (SipProxyException spe) {
                System.err.println("Couldn't init proxy because: "+
                                   spe.toString());
                spe.printStackTrace();
                return null;
            }
        }

        return proxyLookup;
    }

}
