/**
 * A generic utility that will perform actions at certain intervals. This can be
 * repetitive or one-at-a-time.
 *
 * @author S.R.Jones
 * @version 1
 */
package org.mitre.jsip.util;

import java.util.*;

public class Poller
    extends Object {

    // instance variables
    private Timer pollTimer;
    private PollAgent pollAgent;
    private Date lastScheduledEvent;

    public Poller(PollAgent pollAgent) {
        this.pollAgent=pollAgent;
        lastScheduledEvent=new Date();
        pollTimer=new Timer();
    }

    public Poller(PollAgent pollAgent, long delay) {
        this(pollAgent, new Date(), delay);
    }

    public Poller(PollAgent pollAgent, Date startTime, long delay) {
        this.pollAgent=pollAgent;
        pollTimer=new Timer();
        pollTimer.scheduleAtFixedRate(pollAgent, startTime, delay);
    }

    public void schedule(Date time) {
        if (time==null) {
            return;
        }

        synchronized (this) {
            if (!java.text.DateFormat.getDateInstance().format(time).equals(
                      java.text.DateFormat.getDateInstance()
                      .format(lastScheduledEvent))) {
                // make sure previous scheduled job (if any) is cancelled
                pollAgent.cancel();

                lastScheduledEvent=time;
                pollTimer.schedule(pollAgent, time);
            }
        }
    }

    public void cancel() {
        pollTimer.cancel();
    }
}
