/*
*   umasip is a Videoconference program that uses the SIP protocol (RFC 3261)
*   Copyright (C) 2004 Pablo Montoro Escao
*
*   This program is free software; you can redistribute it and/or modify
*   it under the terms of the GNU General Public License as published by
*   the Free Software Foundation; either version 2 of the License, or
*   (at your option) any later version.
*
*   This program is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with this program; if not, write to the Free Software
*   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*
*/
import javax.swing.*;

/**
 * <p>Programa: umasip</p>
 * <p>Copyright: Copyright GNU/GPL (c) 2004 Pablo Montoro Escao</p>
 * @author Pablo Montoro Escao
 * @version 1.00
 * @since SDK 1.4.2.
 */

public class umasip {

    public static final int VERSION=100;//Numero de version del software umasip

    public static final int ESPERA=0;  //Estado de espera de llamadas
    public static final int SONANDO=1; //Estado al recibir una llamada
    public static final int LLAMANDO=2;//Estado al realizar una llamada
    public static final int LLAMADA=3; //Estado de llamada activa

    static int estado=ESPERA; //Estado inicial del programa
    static boolean hayJMF=false;
    static Interfaz interfaz=new Interfaz("umasip");
    static Idioma idioma=interfaz.idioma;
    static Preferencias preferencias;
    static Agente agente;
    static Llamada llamada;

    //Metodo Main o principal del programa
    public static void main(String[] args) {
        int respuesta=JOptionPane.NO_OPTION; //Respuesta a peticiones de llamada

        //Inicializamos el Interfaz
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            e.printStackTrace();
        }
        interfaz.setVisible(true);

        //Comprobamos si se encuentran las clases de JMF
        try {
            Class.forName("javax.media.Processor");
            Class.forName("javax.media.Codec");
            Class.forName("javax.media.rtp.RTPManager");

            //Inicializamos el Agente de Usuario SIP
            preferencias=new Preferencias(idioma);
            agente=new Agente(preferencias.usuario,preferencias.ip);
            agente.audioPT=preferencias.audioPT();
            agente.videoPT=preferencias.videoPT();
            interfaz.setSIP("SIP Protocol Information:\n"+
                            "TIME | Local User ==MESSAGE (Request/Response)"+
                            "== Remote User\n"+agente.getSIP());
            agente.setSIP(interfaz.getSIP());

            hayJMF=true;
        } catch (ClassNotFoundException e) {
            //Inicializamos el Agente de Usuario SIP
            agente=new Agente();
            interfaz.setSIP("JMF not found. You can look for it at:\n"+
                            "http://java.sun.com/products/java-media/jmf\n\n"+
                            "umasip won't work properly!\n"+agente.getSIP());
            agente.setSIP(interfaz.getSIP());
            interfaz.noHayJMF();
            hayJMF=false;
        }

        //Bucle principal del programa
        do {
            if (estado==ESPERA) {
                if (hayJMF && preferencias.solicitaCambio) {
                    interfaz.setSIP(agente.getSIP());
                    agente.salir();
                    agente=new Agente(preferencias.usuario,preferencias.ip);
                    agente.audioPT=preferencias.audioPT();
                    agente.videoPT=preferencias.videoPT();
                    agente.setSIP(interfaz.getSIP()+agente.getSIP());
                    preferencias.solicitaCambio=false;
                }
                if (interfaz.solicitaLlamada) {
                    agente.llamar(interfaz.uriRemoto);
                    llamada=new Llamada(interfaz.uriRemoto,idioma.texto(12));
                    llamada.setImagen("img"+java.io.File.separator+"calling.jpg");
                    //interfaz.solicitaLlamada=false;
                    estado=LLAMANDO;
                } else if (agente.solicitaLlamada) {
                    if (hayJMF) estado=SONANDO;
                    else agente.rechazar();
                } else if (interfaz.solicitaPref) {
                    preferencias.setVisible(true);
                    interfaz.solicitaPref=false;
                }

            } else if (estado==SONANDO) {
                respuesta=interfaz.llamada(agente.remoto);
                if (respuesta==JOptionPane.YES_OPTION) {
                    agente.aceptar();
                    llamada=new Llamada(agente.remoto,idioma.texto(12));
                    llamada.setImagen("img"+java.io.File.separator+"calling.jpg");
                    interfaz.setSIP(agente.getSIP());
                    interfaz.refresh();
                    llamada.TX(agente.ipLocal,agente.ipRemoto,preferencias.audio,
                               agente.paLocal,preferencias.video,agente.pvLocal);
                    interfaz.setSIP(agente.getSIP()+"Initiated Transmission");
                    interfaz.refresh();
                    while ((!llamada.solicitaColgar) &&
                           (!interfaz.solicitaColgar) &&
                           (!agente.solicitaColgar) &&
                           (!llamada.RX(agente.ipLocal,agente.ipRemoto,
                                        agente.paRemoto,agente.pvRemoto))) {
                        interfaz.setSIP(agente.getSIP()+
                                        "Failed initializing the sessions."+
                                        "\nRetrying...");
                        interfaz.refresh();
                    }
                    llamada.setImagen("img"+java.io.File.separator+"call.jpg");
                    estado=LLAMADA;

                } else {
                    agente.rechazar();
                    estado=ESPERA;
                }

            } else if (estado==LLAMANDO) {
                if ( (interfaz.solicitaColgar) || (llamada.solicitaColgar) ) {
                    interfaz.rechazar();
                    agente.colgar();
                    llamada.salir();
                    llamada=null;
                    estado=ESPERA;
                } else if (agente.respuestaRemota==200) {
                    interfaz.aceptar();
                    interfaz.setSIP(agente.getSIP());
                    interfaz.refresh();
                    llamada.TX(agente.ipLocal,agente.ipRemoto,preferencias.audio,
                               agente.paLocal,preferencias.video,agente.pvLocal);
                    interfaz.setSIP(agente.getSIP()+"Initiated Transmission");
                    interfaz.refresh();
                    while ((!llamada.solicitaColgar) &&
                           (!interfaz.solicitaColgar) &&
                           (!agente.solicitaColgar) &&
                           (!llamada.RX(agente.ipLocal,agente.ipRemoto,
                                        agente.paRemoto,agente.pvRemoto))) {
                        interfaz.setSIP(agente.getSIP()+
                                        "Failed initializing the sessions."+
                                        "\nRetrying...");
                        interfaz.refresh();
                    }
                    llamada.setImagen("img"+java.io.File.separator+"call.jpg");
                    estado=LLAMADA;
                } else if (agente.respuestaRemota>200) {
                    llamada.setImagen("img"+java.io.File.separator+"busy.jpg");
                    interfaz.rechazar();
                    agente.colgar();
                    llamada.salir();
                    llamada=null;
                    estado=ESPERA;
                }

            } else if (estado==LLAMADA) {
               if ((interfaz.solicitaColgar) || (llamada.solicitaColgar)) {
                   interfaz.colgar();
                   agente.colgar();
                   llamada.setImagen("img"+java.io.File.separator+"hanging.jpg");
                   llamada.salir();
                   llamada=null;
                   estado=ESPERA;
               } else if (agente.solicitaColgar) {
                   interfaz.colgar();
		   agente.colgar();
                   llamada.setImagen("img"+java.io.File.separator+"hanging.jpg");
                   llamada.salir();
                   llamada=null;
                   estado=ESPERA;
               }

            } else { //CASO ANOMALO
                if (interfaz.solicitaLlamada) {
                    interfaz.solicitaLlamada=false;
                }
                if (agente.solicitaLlamada) {
                    agente.rechazar();
                }

                //Se reinicia el programa por estado desconocido
                if (llamada!=null) {
                    llamada.setImagen("img"+java.io.File.separator+"hanging.jpg");
                    agente.colgar();
                    llamada.salir();
                }
                agente.salir();
                if (hayJMF) {
                    preferencias.salir();
                    preferencias=new Preferencias(idioma);
                    agente=new Agente(preferencias.usuario,preferencias.ip);
                    agente.audioPT=preferencias.audioPT();
                    agente.videoPT=preferencias.videoPT();
                } else {
                    agente=new Agente();
                    interfaz.noHayJMF();
                }
                estado=ESPERA;
            }

            if (interfaz.solicitaTX) {
                preferencias.infoTX();
                interfaz.solicitaTX=false;
            }

            interfaz.setSIP(agente.getSIP());
            interfaz.refresh();
            if (llamada!=null) llamada.repaint();
            try {Thread.sleep(400);} catch(Exception e) {e.printStackTrace();}

        } while (interfaz.fin==false);

        //FIN del programa
        interfaz.setVisible(false);
        if (llamada!=null) {
            llamada.setImagen("img"+java.io.File.separator+"hanging.jpg");
            agente.colgar();
            llamada.salir();
        }
        agente.salir();
        if (hayJMF) preferencias.salir();
        interfaz.salir();
        System.exit(0);
    }
}
